from random import random, sample, choice
from mOTUlizer.classes.mOTU import mOTU, mean
from collections import defaultdict
from numpy.random import normal
from numpy import floor, mean

genome2guass = {}

class MockmOTU(mOTU):
    def __repr__(self) :
        return "< MockmOTU with {n} genomes, of average {c}% completness, with core/genome_len of {r} >".format(c = 100*self.mean_completeness, n = len(self), r = self.ratio)

    def __init__(self, name, core_len, nb_genomes, completeness, max_it = 20, accessory = None, method = None):

        core = {"CoreTrait_{}".format(i) for i in range(core_len)}

        if accessory is None:
            sub_dist = [int(nb_genomes/i) for i in range(2,1000) if int(nb_genomes/i) > 0] + [1]*100
            sub_dist = list(range(nb_genomes-1, 1,-1))
        else :
            sub_dist = accessory

        self.size_accessory = sum(sub_dist)
        self.mean_size_accessory = sum(sub_dist)/nb_genomes

        mock_genomes = dict()
        for k in range(nb_genomes):
            mock_genomes["Genome_{}".format(k)] = list(core)

        for i,v in enumerate(sub_dist):
            genomes = sample(list(mock_genomes.keys()), v if v < len(mock_genomes) else len(mock_genomes) )
            for g in genomes:
                mock_genomes[g] += ["AccessoryTrait_{}".format(i)]

        self.incompletes = {g : {vv for vv in v if random() < (completeness(g)/100)} for g, v in mock_genomes.items()}

        to_rm = []
        for k, v in self.incompletes.items():
            if len(v) == 0:
                if len(core) > 0:
                    self.incompletes[k] = choice(list(core))

        if core_len == 0:
            self.mean_completeness = "NA"
            self.completenesses = {c : 0 for c,v in self.incompletes.items()}
        else :
            self.mean_completeness = mean([len({vv for vv in v if vv.startswith("CoreTrait_")})/core_len for c,v in self.incompletes.items()])
            self.completenesses = {c : 100*len({vv for vv in v if vv.startswith("CoreTrait_")})/core_len for c,v in self.incompletes.items()}
    #        self.accessory = accessory
        self.mean_size = mean([len(m) for m in mock_genomes.values()])
        self.real_core_len = core_len

        zerifneg = lambda g: 0.001 if g < 0 else g
        super().__init__(name = name, faas = {}, cog_dict = self.incompletes, checkm_dict = { k : zerifneg(normal(v, 10)) for k,v in self.completenesses.items()}, max_it = max_it, method = method, quiet=True)
        if core_len == 0:
            self.recall = "NA"
            self.fpr = "NA"
        else :
            self.recall = len(core.intersection(self.core))/core_len
            self.fpr = sum([not c.startswith("CoreTrait_") for c in self.core])/len([not c.startswith("CoreTrait_") for c in self.core])

        self.lowest_false = {k : v for k,v in self.cogCounts.items() if k in self.core and k not in core}
        self.lowest_false = 1 if(len(self.lowest_false) ==0) else min(self.lowest_false.items(), key = lambda x : x[1])[1]/len(self)


    def mock_cog_stats(self):
        all_genes = set.union(*self.incompletes.values())
        outp = {t : {} for t in all_genes}
        for t,dd  in outp.items():
            dd['freq'] = sum([t in zz for zz in self.incompletes.values()])/len(self.incompletes)
            dd['core'] = t in self.core
            dd['type'] = "core" if t.startswith("CoreTrait_") else "accessory"
            dd['nb_genomes'] = len(self)
            dd['core_len'] = len(self.core)
            dd['real_core_len'] = self.read_core_len
            dd['llikelihood'] = self.likelies[t]
            dd['len_accessory_genome'] = len(all_genes) - dd['real_core_len']
        return outp

    @classmethod
    def guauss_completes(cls, g, mean_completeness = 60, stdev = 10):
        if g in genome2guass:
            return genome2guass[g]
        else :
            out_prob = 1000
            while(not (20 < out_prob < 99) ):
                out_prob = normal(mean_completeness, stdev)
            genome2guass[g] = out_prob
            return out_prob

    @classmethod
    def run_boots(cls):
        out = {}

        for i in range(1000, 2500, 500):
            for nb in range(10, 250, 15):
                for c in range(30, 100, 5):
                    MockData.genome2guass = {}
                    mockmotu = MockmOTU("complete_{}_core_size_{}_nbgenomes_{}".format(c,i,nb).format(c), i, nb, lambda g : MockmOTU.guauss_completes(g, mean_completeness = c, stdev = 10), max_it = 100)
                    out[mockmotu.name] = {}
                    out[mockmotu.name]['core_size'] = i
                    out[mockmotu.name]['nb_genomes'] = nb
                    out[mockmotu.name]['mean_completeness'] = mockmotu.mean_completeness
                    out[mockmotu.name]['mean_genome_size'] = mockmotu.mean_size
                    out[mockmotu.name]['recall'] = mockmotu.recall
                    out[mockmotu.name]['lowest_false'] = mockmotu.lowest_false
                    out[mockmotu.name]['accessory_genepool'] = mockmotu.size_accessory
                    out[mockmotu.name]['mean_new_completness'] = mean([b.new_completness for b in mockmotu])
        return out



#!/usr/bin/env python

import os
import shutil
import sys
from os.path import join as pjoin
import argparse
import json
from random import uniform

#print("This is temporary, fix the hard-path once all is clean", file=sys.stderr)
#sys.path.append("/home/moritz/projects/0039_mOTUlizer/")

from mOTUlizer import __version__
from mOTUlizer.classes import *
from mOTUlizer.utils import *
from mOTUlizer.classes.mOTU import mOTU
import multiprocessing

#from mOTUlizer.config import *

description_text = """
From a buch of amino-acid sequences or COG-sets, computes concensus AA/COG sets.

Returns all to stdout by default.
"""
__checkm_default__ = 95

def motupan(args):
    if args.cog_file:
        try :
            with open(args.cog_file) as handle:
                cog_dict = json.load(handle)
        except :
            try :
                with open(args.cog_file) as handle:
                    cog_dict = {l.split("\t")[0] : l[:-1].split("\t")[1:] for l in handle}
            except :
                print("Either the cog_file does not exists or it is not formated well", file = sys.stderr)
        cog_dict = {k : set(v) for k,v in cog_dict.items()}
        if all([len(v) == 0 for v in cog_dict]):
            print("None of your bins have any cogs in them, that's weird, you probably have wrong delimiter in you file, use tab.\nIf you do not have COGs you can also just run it without the --cog_file option and mOTUlizer will automatically compute some!", file = sys.stderr)
    else :
        cog_dict = {}

    #parse and check your amino-acid files
    if args.txt and args.faas:
        with open(args.faas[0]) as handle:
            faas = {os.path.splitext(os.path.basename(f.strip().rstrip(".gz")))[0] : f.strip() for f in handle.readlines()}
    elif args.faas:
        faas = {os.path.splitext(os.path.basename(f.strip().rstrip(".gz")))[0] : f for f in args.faas}
    else :
        faas = {}

    assert all([os.path.exists(f) for f in faas.values()]), "one or some of your faas don't exists"


    if len(faas) > 0 and args.cog_file:
        genomes = set(faas.keys()).intersection(set(cog_dict.keys()))
    elif len(faas) > 0:
        genomes = set(faas.keys())
    else :
        genomes = set(cog_dict.keys())

    if cog_dict and len(faas) > 0:
        if len(genomes) != len(faas) or len(faas) != len(cog_dict):
            print("your faas and cog_drct are not the same length,\nit might not matter just wanted to let you know.", file = sys.stderr)

    if len(cog_dict) > 0 :
        cog_dict = {g : cog_dict.get(g) for g in genomes if g in cog_dict}

    out_json = args.output
    checkm = {}
    if args.checkm :
        assert os.path.exists(args.checkm), "The file for checkm does not exists"

        print("Parsing the checkm-file", file = sys.stderr)
        checkm = {k : v['Completeness'] for k,v in parse_checkm(args.checkm).items()}
        checkm = {g : checkm.get(g) for g in genomes}
        if not all([v for v in checkm.values()]):
            print("you did not have completeness for all bins/genomes, abscent values have been put to default value, e.g. " + str(__checkm_default__), file = sys.stderr)
            checkm = {k : v if v  else __checkm_default__ for k,v in checkm.items()}
    elif args.length_seed :
        checkm = "length_seed"
    elif args.seed :
        for f in genomes:
            checkm[f] = args.seed
    elif args.random_seed :
        for f in genomes:
            checkm[f] = uniform(50,80)
    else :
        for f in genomes:
            checkm[f] = __checkm_default__

    threads = args.threads
    precluster = args.precluster
    method = None if args.genome2cog_only else "default"

    name = args.name if args.name else random_name()
    name = (name + "_") if not name.endswith("_") else name
    max_it = args.max_iter
    if faas is None and cogs is None:
        sys.exit("at least one of --faas and --cog_file is required")

    print(len(genomes), " genomes in your clade", file = sys.stderr)

    motu = mOTU( name = name , faas = faas , cog_dict = cog_dict, checkm_dict = checkm, max_it = max_it, threads = threads, precluster = precluster, method = method)

    if args.output:
        out_handle = open(out_json, "w")
    else :
        out_handle = sys.stdout
    stats = motu.get_stats()

    nb_boots = args.boots
    if args.long and not args.genome2cog_only:

        stats[name].update(motu.roc_values(boots=nb_boots))
        json.dump(stats, out_handle, indent=4, sort_keys=True)
    elif args.genome2cog_only :
        json.dump({k : list(v) for k,v in motu.cog_dict.items()}, out_handle, indent=4, sort_keys=True)
    else :
        out_dict = {}

        stats = list(stats.values())[0]
        stats.update(motu.roc_values(boots = nb_boots))
        cogs = set([cc for g,c in stats['cogs'].items() for cc in c]) if 'aa' not in stats['cogs'] else set(stats['cogs']['aa'].values())
        for k in cogs:
            out_dict[k] = {}
            out_dict[k]['type'] = 'core' if k in stats['core'] else 'accessory'
            out_dict[k]['genome_occurences'] = 0
            out_dict[k]['log_likelihood_to_be_core'] = stats['likelies'][k]
            out_dict[k]['genomes'] = []
            out_dict[k]['genes'] = [] if 'aa' in stats['cogs'] else ["NA"]
            out_dict[k]['trait_name'] = k
        if 'aa' in stats['cogs']:
            for k,v in stats['cogs']['aa'].items():
                out_dict[v]['genes'] += [k]
        for k,v in stats['cogs'].items() if 'aa' not in stats['cogs'] else stats['cogs']['genome'].items():
            for vv in v:
                out_dict[vv]['genomes'] += [k]
                out_dict[vv]['genome_occurences'] += 1

        for k,v in out_dict.items():
            v['mean_copy_per_genome'] = "NA" if not v['genes'] else len(v['genes'])/len(v['genomes'])
            v['genes'] = ";".join(v['genes'])
            v['genomes'] = ";".join(v['genomes'])

        header = ['trait_name','type', 'genome_occurences', 'log_likelihood_to_be_core', 'mean_copy_per_genome','genomes', 'genes']
        genome_line = "genomes=" + ";".join(["{}:prior_complete={}:posterior_complete={}".format(k['name'], k['checkm_complet'], k['new_completness']) for k in stats['genomes']])
        mean = lambda l : sum([ll for ll in l])/len(l)

        if stats['mean_recall'] != "NA":
            bootsy = """
#bootstrapped_mean_false_positive_rate={fpr:.2f};bootstrapped_sd_false_positive_rate={sd_fpr:.2f}
#bootstrapped_mean_recall={recall:.2f};bootstrapped_sd_recall={sd_recall:.2f}
#bootstrapped_mean_lowest_false_positive={lowest:.2f};bootstrapped_sd_lowest_false_positive={sd_lowest:.2f}
#bootstrapped_nb_reps={boots}
#""".format( boots=nb_boots,
            fpr=stats['mean_fpr'],
            recall = stats['mean_recall'], lowest = stats['mean_lowest_false'],sd_fpr=stats['sd_fpr'],
            sd_recall = stats['sd_recall'], sd_lowest = stats['sd_lowest_false'])
        else :
            bootsy=""


        outformat ="""#mOTUlizer:mOTUpan:{version}
#run_name={name}
#
#genome_count={nb}
#core_length={core_len}
#mean_prior_completeness={prior_complete:.2f}
#mean_posterior_completeness={post_complete:.2f}
#sum_abs_loglikelihood_ratios={SALLHR:.2f}
#mean_est_genome_size={size:.2f};traits_per_genome
#{genomes}
#{boostrap}
{header}
{data}
"""
        print(outformat.format(version = __version__ , nb = stats['nb_genomes'],
            name = motu.name.strip("_"),
            core_len = len(motu.core),
            genomes=genome_line,
            prior_complete=mean([b.checkm_complet for b in motu]),
            post_complete=mean([b.new_completness for b in motu]),
            SALLHR=sum([l if l > 0 else -l for l in motu.likelies.values()]),
            size=mean([100*len(b.cogs)/b.new_completness for b in motu]),
            boostrap=bootsy,
            header = "\t".join(header), data = "\n".join(["\t".join([str(v[hh]) for hh in header]) for v in out_dict.values()])),
            file=out_handle)

    if args.output:
        out_handle.close()

    return None

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog = "mOTUpan.py", description=description_text, epilog = "Let's do this")
    parser.add_argument('--output', '-o', nargs = None, help = "send output to this file")
    parser.add_argument('--force', '-f', action='store_true', help = "force execution answering default answers")
    parser.add_argument('--checkm', '-k',nargs = None, help = "checkm file if you want to seed completnesses with it, accepts concatenations of multiple checkm-files, check manual for more formating options")
    parser.add_argument('--seed', '-s', type = float , nargs = '?', help = "seed completeness, advice a number around 90 ({} default), this is the default completeness prior".format(__checkm_default__))
    parser.add_argument('--length_seed', '--ls', action='store_true', help = "seed completeness by length fraction [0-100]")
    parser.add_argument('--random_seed', '--rs', action='store_true', help = "random seed completeness between 5 and 95 percent")
    parser.add_argument('--genome2cog_only', action='store_true', help = "returns genome2cog only")
    parser.add_argument('--precluster', action='store_true', default = False, help = "precluster proteomes with cd-hit, mainly for legacy reasons, mmseqs2 is faaaaaast")
    parser.add_argument('--faas','-F', nargs = '*', help = "list of amino-acids faas of MAGs or whatnot, or a text file with paths to the faas (with the --txt switch)")
    parser.add_argument('--txt', action='store_true', help = "the '--faas' switch indicates a file with paths")
    parser.add_argument('--cog_file', '--cogs', '-c', nargs = '?', help = "file with COG-sets (see doc)")
    parser.add_argument('--name', '-n', nargs = None, help = "if you want to name this bag of bins")
    parser.add_argument('--long', action='store_true', help = "longform output, a JSON-file with a lot more information (might be cryptic...)")
    parser.add_argument('--boots', '-b', nargs = None, type = int , default = 0 , help = "number of bootstraps for fpr and recall estimate (default 0), careful, slows down program linearly")
    parser.add_argument('--max_iter', '-m', nargs = None, type = int , default = 20 , help = "max number of iterations (set to one if you have only few traits, e.g. re-estimation of completeness is nonsensical)")
    parser.add_argument('--threads', '-t', nargs = None, type = int , default = multiprocessing.cpu_count(), help = "number of threads (default all, e.g. {}), only gene clustering is multithreaded right now, the rest is to come".format(multiprocessing.cpu_count()))
    parser.add_argument('--version','-v', action="store_true", help = "get version")

    args = parser.parse_args()

    if len(sys.argv)==1:
        parser.print_help(sys.stderr)
        print("{script} Version {version}".format(script = __file__, version = __version__), file = sys.stderr)
        sys.exit(1)

    if args.version:
        print("{script} Version {version}".format(script = __file__, version = __version__), file = sys.stderr)
        sys.exit()


#    print(args, file=sys.stderr)

    motupan(args)


#for tt in `sed 's/\t/:/' scratch/test_data/mOTUs.txt` ;
#do
#    echo $tt | cut -f1 -d":"
#    fs=`echo $tt | cut -f2 -d":"| sed 's#;#.faa scratch/test_data/proteoms/#g'`;
#    mOTUlizer/bin/__main__.py -n `echo $tt | cut -f1 -d":"` --faas scratch/test_data/proteoms/${fs}.faa >> test;
#done



#!/usr/bin/env python

import os
import shutil
import sys
from os.path import join as pjoin
import argparse
import json
from random import uniform
import multiprocessing
from mOTUlizer.classes import *
from mOTUlizer.utils import *
from mOTUlizer.config import FASTA_EXTS
from mOTUlizer.classes.MetaBin import MetaBin
from mOTUlizer.classes.mOTU import mOTU
from mOTUlizer import __version__

#from mOTUlizer.config import *

description_text = """
From a set of genomes, makes metagenomic Operational Taxonomic Units (mOTUs). By default it makes a graph of 95%
(reciprocal) ANI (with fastANI) connected MAGs (with completeness > 40%, contamination < 5%). The mOTUs will be the
connected components of this graph, to which smaller "SUBs" with ANI > 95%  are
recruited.

If similarities provided, it should be a TAB-separated file with columns as query, subject and similarity (in percent, e.g. [0-100])
if you also provide fasta-files (for stats purpouses) query and names should correspond to the fasta-files you provide.
If the columns are file names, the folders are removed (mainly so it can read fastANI output directly).

"""


def motulize(args):
    #parse and check your amino-acid files

    if args.txt and args.fnas:
        with open(args.fnas[0]) as handle:
            fnas = {os.path.splitext(os.path.basename(f.strip().rstrip(".gz")))[0] : f.strip() for f in handle.readlines()}
    elif args.fnas:
        fnas = {os.path.splitext(os.path.basename(f.strip().rstrip(".gz")))[0] : f for f in args.fnas}
    else :
        fnas = {}

    ani_cutoff = args.similarity_cutoff
    similarities = args.similarities
    checkm_file = args.checkm
    out_json = args.output
    prefix = args.prefix
    mag_complete = args.MAG_completeness
    mag_contamin = args.MAG_contamination
    sub_complete = args.SUB_completeness
    sub_contamin = args.SUB_contamination
    force = args.force
    threads = args.threads
    keep_simi = args.keep_simi_file


    assert 0 < ani_cutoff < 100, "similarity cutoff needs to be between 0 and 100 (percent similarity)"
    assert all([os.path.exists(f) for f in fnas.values()]), "one or some of your fnas don't exists"
    assert not (len(fnas) == 0 and similarities is None), "you need to give at least either a list of genome FASTA-files ('--fnas' option) or a file with pariwise similarities ('--similarities' option)"

    if similarities:
        assert os.path.exists(similarities), "The file for similarities does not exists"
        keep_simi = None
    if checkm_file:
        assert os.path.exists(checkm_file), "The file for checkm does not exists"


    if similarities:
        print("Loading similarities", file = sys.stderr)
        dist_dict = {}
        with open(similarities) as handle:
            for l in handle:
                if "query" not in l:
                    ll = l.split("\t")
                    g1 = ".".join(os.path.basename(ll[0]).split(".")[:-1]) if any([ll[0].endswith(ext) for ext in FASTA_EXTS]) else ll[0]
                    g2 = ".".join(os.path.basename(ll[1]).split(".")[:-1]) if any([ll[1].endswith(ext) for ext in FASTA_EXTS]) else ll[1]
                    dist = float(ll[2])
                    dist_dict[(g1,g2)] = dist
    else :
        dist_dict = None

    if len(fnas) > 0 :
        genomes = set(fnas.keys())
    else :
        genomes = {a for k in dist_dict for a in k}

    if dist_dict:
        print("{nb_gen} genomes found with {counts} ANI mOTU edges".format(nb_gen = len(genomes), counts = len(dist_dict)), file=sys.stderr)
    else :
        print("{nb_gen} genomes found".format(nb_gen = len(genomes)), file=sys.stderr)

    print("Parsing the completeness/redundancy-file", file = sys.stderr)

    if checkm_file is None:
        print("No file provided, all genomes are assumed perfect (100% complete, 0% redundancy)", file=sys.stderr)
        checkm_info = {g : {'Completeness' : 100, 'Contamination' :0} for g in genomes}
    else :
        checkm_info = parse_checkm(checkm_file)


    assert all([g in checkm_info  for g in genomes]), "you do not have completness/contamination info for all you bins, values missing for :" + ", ".join([g for g in genomes if g not in checkm_info][0:10]) + "... (only 10 first shown   )"

    if fnas == {}:
        fnas = {g : None for g in genomes}

    print("making bin-objects", file = sys.stderr)

    all_bins = [MetaBin(name = g, cogs = None, faas = None, fnas = fnas[g], complet = checkm_info[g]['Completeness'], contamin = checkm_info[g]['Contamination'], max_complete = 100) for g in genomes]

    if dist_dict is None:
        print("Similarities not provided, will compute them with fastANI", file = sys.stderr)
        if keep_simi and not force:
            assert not os.path.exists(keep_simi), "similarity file already exists, delete it or use '--force'"

        dist_dict = MetaBin.get_anis(all_bins, threads = threads, outfile = keep_simi)

    print("making mOTUs", file = sys.stderr)

    mOTUs = mOTU.cluster_MetaBins(all_bins, dist_dict, ani_cutoff, prefix, mag_complete, mag_contamin, sub_complete, sub_contamin)

    print("making stats", file = sys.stderr)

    out_dict = {}
    for m in mOTUs:
        stats = m.get_stats()
        stats[m.name]['representative'] = m.get_representative()
        out_dict.update(stats)

    short_out = []
    header = ['mOTU', 'representative', 'mean_ANI', 'min_ANI', 'missing_edges', 'MAGs', 'SUBs']
    for k, v in out_dict.items():
        row = {
        'mOTU' : k,
        'representative' : v['representative'],
        'mean_ANI' : v['mean_ANI']['mean_ANI'],
        'min_ANI' : min([vv[2] for vv in v['ANIs']]),
        'missing_edges' : v['mean_ANI']['missing_edges'],
        'MAGs' : ";".join([g['name'] for g in v['genomes'] if g['checkm_complet'] > mag_complete and g['checkm_contamin'] < mag_contamin]),
        'SUBs' : ";".join([g['name'] for g in v['genomes'] if g['checkm_complet'] <= mag_complete or g['checkm_contamin'] >= mag_contamin])

        }
        short_out += [row]

    genome_line = "genomes=" + ";".join(["{}:completeness={}:redundancy={}".format(k['name'], k['checkm_complet'], k['checkm_contamin']) for v in out_dict.values() for k in v['genomes'] ])

    outformat ="""#mOTUlizer:mOTUlize:{version}
#prefix={name}
#
#{genomes}
#
{header}
{data}"""


    if args.output:
        if not force:
            assert not os.path.exists(args.output), "output file already exists, delete it or use '--force'"
        out_handle = open(out_json, "w")
    else :
        out_handle = sys.stdout
    if args.long:
        json.dump(out_dict, out_handle, indent=4, sort_keys=True)
    else :
        print(outformat.format(version = __version__ ,
            name = prefix,
            genomes=genome_line,
            header = "\t".join(header),
            data = "\n".join(["\t".join([str(v[hh]) for hh in header]) for v in short_out])),
            file=out_handle)
    if args.output:
        out_handle.close()

    return None

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog = "mOTUlize", description=description_text, epilog = "Let's do this")
    parser.add_argument('--output', '-o', nargs = '?', help = "send output to this file")
    parser.add_argument('--force', '-f', action='store_true', help = "force execution (overwritting existing files)")
    parser.add_argument('--checkm', '-k',nargs = '?', help = "checkm file (or whatever you want to use as completness), if not provided, all genomes are assumed to be seed MAG (e.g complete enough)")
    parser.add_argument('--similarities', '-I', nargs = '?', help = "file containing similarities between MAGs, if not provided, will use fastANI to compute one")
    parser.add_argument('--fnas','-F', nargs = '*', help = "list of nucleotide fasta-files of MAGs or whatnot")
    parser.add_argument('--prefix', '-n', nargs = '?', default = "mOTU_", help = "prefix for the mOTU names, default : mOTU_ ")
    parser.add_argument('--MAG-completeness', '--MC', '-M', nargs = '?', type=float, default = 40, help = "completeness cutoff for seed MAGs, default : 40")
    parser.add_argument('--MAG-contamination', '--Mc', '-m', nargs = '?', type=float, default = 5, help = "contamination cutoff for seed MAGs, default : 5")
    parser.add_argument('--SUB-completeness', '--SC', '-S', nargs = '?', type=float, default = 0, help = "completeness cutoff for recruited SUBs, default : 0")
    parser.add_argument('--SUB-contamination', '--Sc', '-s', nargs = '?', type=float, default = 100, help = "contamination cutoff for recruited SUBs, default : 100")
    parser.add_argument('--similarity-cutoff', '-i', nargs = '?', type=float, default = 95, help = "distance cutoff for making the graph, default : 95")
    parser.add_argument('--threads', '-t', nargs = None, type = int , default = multiprocessing.cpu_count(), help = "number of threads (default all, e.g. {})".format(multiprocessing.cpu_count()))
    parser.add_argument('--keep-simi-file', '-K', nargs = '?', default = None, help = "keep generated similarity file if '--similarities' is not provided, does nothing if '--similarity' is provided")
    parser.add_argument('--txt', '-T', action='store_true', help = "the '--fnas' switch indicates a file with paths")
    parser.add_argument('--long', action='store_true', help = "longform output, a JSON-file with a lot more information (might be cryptic...)")
    parser.add_argument('--version','-v', action="store_true", help = "get version")

    args = parser.parse_args()

    if len(sys.argv)==1 or args.version:
        if len(sys.argv)==1:
            parser.print_help(sys.stderr)
        print("{script} Version {version}".format(script = __file__, version = __version__))
        sys.exit(1)

    motulize(args)



import random
import string
import re

def random_name(stringLength=8):
    """Generate a random string of fixed length """
    letters = string.ascii_uppercase
    return ''.join(random.choice(letters) for i in range(stringLength))


def parse_checkm(file):
    # Should be able to read old checkm files, new checkm files  (e.g. checkm lineage_wf infolder datfolder > checkm_file)
    # or any TAB-serparated files with a column called 'Bin Id', 'Completeness', and 'Contamination'
    with open(file) as handle:
        all_lines = [l.strip() for l in  handle.readlines() if " INFO:" not in l]

    all_lines = [re.sub(r"  +","\t", a).split("\t") for a in all_lines]

    header_lines = [i for i,l in enumerate(all_lines) if 'Bin Id' in l and 'Completeness' in l and 'Contamination' in l]
    assert len(header_lines) >= 1, "your completness files is badly formed, it should be TAB-separated (multispaced...) and needs a header line with 'Bin Id', 'Completeness', and 'Contamination' in it"

    header_lines = header_lines[0]
    header_line = all_lines[header_lines]

    lines = [l for i,l in enumerate(all_lines) if i != header_lines and len(l) == len(header_line) and 'Completeness' not in l]

    assert len(lines) > 0, "there are no lines in the file, you sure the serparators are 'OK'"

    lines = [{a : b if a not in ['Completeness', 'Contamination'] else float(b) for a,b in zip(header_line,l) }for l in lines]
    return {l['Bin Id'] : l for l in lines}



import tempfile
import sys, os
import gzip
from mOTUlizer.config import *
from Bio import SeqIO
from os.path import join as pjoin
import shutil

methods =  ['silixCOGs', 'mmseqsCluster']

def compute_COGs(faas, name, precluster = False, threads = 4, method =  "mmseqsCluster"):
    name = name + method + "_"

    temp_folder = tempfile.mkdtemp(prefix = name)
    all_faas_file = pjoin(temp_folder, "concat.faa")
    cog2rep = None
    prot_ids = set()
    prot2faa ={}
    for k,v in faas.items():
        if not v.endswith(".gz"):
            with open(v) as handle:
                ids = [l[:-1].split()[0][1:] for l in handle if l[0] == ">"]
        else :
            with gzip.open(v) as handle:
                ids = [l.decode()[:-1].split()[0][1:] for l in handle if l.decode()[0] == ">"]
        for i in ids:
            if i in prot2faa:
                prot2faa[i] += [k]
            else :
                prot2faa[i] = [k]

    print("concatenating all faas", file = sys.stderr)

    if all([faa.endswith(".gz") for faa in faas.values()]):
        cat = "zcat "
    elif all([not faa.endswith(".gz") for faa in faas.values()]) :
        cat = "cat "
    else :
        sys.exit("Please make my life easy, either all faas gziped or none gzipped ...")

    os.system(cat + " ".join([*faas.values()]) + " > " + all_faas_file)

    if precluster:
        cdhit_file = tempfile.NamedTemporaryFile().name

        if not shutil.which('cd-hit'):
            print("You need cd-hit to run the preclustering, either install it or run mOTUpan without preclustering", file = sys.stderr)
            sys.exit(-1)

        exec = "cd-hit -i {input} -o {output} -c 0.95 -M 0 -T {threads} -d 0 -s 0.95 >> {log} 2>&1".format(input = all_faas_file, output = cdhit_file, threads = threads, log = "/dev/null")
        print("Running cd-hit preclustering", file = sys.stderr)
        os.system(exec)

        print("parsing cd-hit", file = sys.stderr)

        with open(cdhit_file + ".clstr") as handle:
            clusters = "\n".join(handle.readlines()).split("Cluster ")

        os.remove(cdhit_file + ".clstr")
        os.remove(cdhit_file)

        clusters = [c.split("\n\n") for c in clusters[1:] if "*" in c]
        clusters = [[cc.split(">")[1].split("... ") for cc in c if ">" in cc and cc != ">"] for c in clusters ]
        clusters = {[cc[0] for cc in c if cc[1] == "*" or cc[1] == "*\n"][0] : [cc[0] for cc in c] for c in clusters}

        print("For", len(prot2faa), "CDSes we got ", len(clusters), "preclusters", file = sys.stderr)
        seqs = [s for s in SeqIO.parse(all_faas_file, "fasta") if s.id in clusters]
        SeqIO.write(seqs, all_faas_file, "fasta")

    if method == "silixCOGs" :
        if not shutil.which("diamond") or not shutil.which("silix"):
            print("You need diamond and silix to run the silix gene-clustering, either install it or run mOTUpan with an other gene-clustering or your own traits", file = sys.stderr)
            sys.exit(-1)

        print("all v all diamond for silix", file = sys.stderr)

        os.system("diamond makedb --db {faas} --in {faas} > /dev/null 2> /dev/null".format(faas = all_faas_file))
        os.system("diamond blastp --more-sensitive -p {threads} -f 6 -q {faas} --db {faas} -o {out} 2> /dev/null > /dev/null".format(faas = all_faas_file, out = temp_out, threads = threads))

        print("running silix", file = sys.stderr)
        os.system("silix {faas} {out} > {clust_temp} #2> /dev/null".format(faas = all_faas_file, out = temp_out, clust_temp = temp_clust))

        print("parsing silix", file = sys.stderr)
        with open(temp_clust) as handle:
            if precluster:
                recs = {g : l[:-1].split()[0]  for l in handle for g in clusters[l[:-1].split()[1]]}
            else :
                recs = {l[:-1].split()[1] : l[:-1].split()[0]  for l in handle}


        #pretty formating names
        fill = max([len(v) for v in recs.values()])
        recs = {k : name + "_" + v.zfill(fill) for k, v in recs.items()}

        genome2cog = {k : set() for k in faas.keys()}
        for k,v in recs.items():
            for vv in prot2faa[k]:
                genome2cog[vv].update([v])

    elif method == "mmseqsCluster" :
        "coverage = 80% with cov-mode = 0, minimal amino acid sequence identity = 80% and cluster-mode = 0"
        covmode = 0
        cov = 0.80
        seqid = 0.80
        if not shutil.which("mmseqs"):
            print("You need mmseqs2 to run the silix gene-clustering, either install it or run mOTUpan with an other gene-clustering or your own traits", file = sys.stderr)
            sys.exit(-1)

        print("running mmseqs easy-cluster with params --min-seq-id {seqid} --cov-mode {covmode} -c {cov} in {}".format(temp_folder, covmode = covmode, cov = cov, seqid=seqid), file = sys.stderr)
        mmseqs_dat = pjoin(temp_folder, "mmseqs_")
        os.system("mmseqs easy-cluster --threads {threads} --min-seq-id {seqid} --cov-mode {covmode} -c {cov} {faas} {out} {tmp} 2> /dev/null > /dev/null".format(covmode = covmode, cov = cov, seqid=seqid, faas = all_faas_file, out = mmseqs_dat, tmp = temp_folder, threads = threads))

        shutil.copy(mmseqs_dat + "_cluster.tsv", ".")
        with open(mmseqs_dat + "_cluster.tsv") as handle:
            if precluster:
                recs = {g : l[:-1].split()[0]  for l in handle for g in clusters[l[:-1].split()[1]]}
            else :
                recs = {l[:-1].split()[1] : l[:-1].split()[0]  for l in handle}

        #pretty formating names
        fill = len(str(len(set(recs.values()))))

        rep2clust = {k : name + str(i).zfill(fill) for i,k in enumerate(set(recs.values()))}
        cog2rep = {v: k for k,v in rep2clust.items()}

        print("For", len(recs), "CDSes we got ", len(cog2rep), " gene-clusters", file = sys.stderr)

        recs = {k : rep2clust[v] for k, v in recs.items()}
        genome2cog = {k : set() for k in faas.keys()}
        for k,v in recs.items():
            for vv in prot2faa[k]:
                genome2cog[vv].update([v])

    else :
        print("The '{}' clustering method is not implemented yet".format(name) , file = sys.stderr)
        print("only allowed are :", methods, file = sys.stderr)
        sys.exit(-1)


    shutil.rmtree(temp_folder)

    for k,v in genome2cog.items():
        genome2cog[k] = set(v)



    return { 'genome2cogs' : genome2cog, 'aa2cog' : recs, 'cog2rep' : cog2rep}



from os.path import join as pjoin
import sys, os
from Bio import Entrez, SeqIO
from xml.etree import ElementTree as ET
from tqdm import tqdm
import re
from collections import defaultdict
import gzip
import json
from random import choices
from mOTUlizer import __version__
from mOTUlizer.classes import *
from mOTUlizer.utils import *
from mOTUlizer.classes.mOTU import mOTU
from multiprocessing import Pool
from statistics import mean
import h5py
import hdf5plugin
import pandas
import shutil
from subprocess import call
from numpy import mean, median
from random import sample


folder = "/home/moritz/projects/0039_mOTUlizer/test_data/prochlos/"
os.chdir(folder)
stratfresh = pandas.read_csv("/home/moritz/data/data_submit/metadata/master_table.csv", index_col=0)
#stratfresh_motus = pandas.read_csv("/home/moritz/data/data_submit/metadata/Supplementary_Table_S7_-_Data_about_mOTUs.csv", index_col=0)
#stratfresh_motus["est_size"] = list(100*stratfresh.loc[stratfresh_motus.representative_MAGs].length/stratfresh.loc[stratfresh_motus.representative_MAGs].completeness/1000000)
with open("stratfreshmotus.json") as handle :
    stratfresh_motus = json.load(handle)

with open("stratfreshmotus.json") as handle :
    bin2stratfreshmotu = { g['name'] : k for k,v in tqdm(json.load(handle).items()) for g in v['genomes']  }

with open("stratfreshmotus.json") as handle :
    stratfreshmotu2bin = { k: [ g['name'] for g in v['genomes']] for k,v in tqdm(json.load(handle).items())  }

counts_sfdb = {k : len(v) for k,v in stratfreshmotu2bin.items()}


sfdb_ids = {k : v for k,v in stratfreshmotu2bin.items() if len(v) > 20}

r95 = pandas.read_csv("/home/moritz/data/gtdb/bac120_metadata_r95.tsv", sep ="\t", index_col=0)
r95['est_size'] = 100*r95.genome_size/r95.checkm_completeness/1000000

gtdb2rep = {v[1]['gtdb_taxonomy'] : v[1]['gtdb_genome_representative'] for v in  r95[['gtdb_genome_representative', 'gtdb_taxonomy']].iterrows() }

counts = {s : 0 for s in set(r95.gtdb_taxonomy)}
for s in r95.gtdb_taxonomy:
     counts[s] += 1
gids = {s : [] for s,v in counts.items() if v > 20}

for k,v in r95.gtdb_taxonomy.items():
     if v in gids:
         gids[v] += [k]



def make_folder(gid, k):
    gid = gid[3:]
    pat = pjoin("/home/moritz/data/gtdb/root/{k}/{gid}/{gid}.fna.gz".format(k = k.replace(";","/"),gid = gid))
    return pat

def make_folder2(gid, ass, path):
    pat = pjoin("{path}/{ass}/{gid}.tar.gz".format(path = path, ass = ass,gid = gid))
    return pat


gids = {k: [l for l in ll if os.path.exists(make_folder(l, k))] for k, ll in tqdm(gids.items()) if sum([os.path.exists(make_folder(l, k)) for l in ll])  > 20}

split_gids = { k : {'goods' : [vv for vv in v if r95.loc[vv,"checkm_completeness"] > 90], 'bads' : [vv for vv in v if r95.loc[vv,"checkm_completeness"] < 90] }for k,v in gids.items()}
split_gids = {k : v  for k,v in split_gids.items() if  (len(v['goods']) > 10) and len(v['bads']) > 10}
ccs = {k : min([len(vv) for vv in v.values()])  for k,v in split_gids.items()}

split_gids = {k : {'goods' : v['goods'][:ccs[k]], 'bads' : v['bads'][:ccs[k]]}   for k,v in split_gids.items()}

def process_species(liss, tax, sett ="gtdb", subset = None):
    print("prepping files")
    if subset:
        liss = sample(liss, subset)
    if os.path.exists("temp/"):
        shutil.rmtree("temp/")
    if sett == "gtdb" :
        paths = [make_folder(g, tax) for g in liss]
        paths = [g for g in paths if os.path.exists(g)]
        os.makedirs("temp", exist_ok = True)
        for p in paths:
            shutil.copy(p, "temp/")
        paths = [pjoin("temp", g) for g in os.listdir("temp") ]
        checkm_dict = { "_".join(k.split("_")[1:]) : r95.loc[k, "checkm_completeness"]  for k in liss}
    if sett == "stratfreshdb":
        pat = "/home/moritz/data/data_submit/bins"
        paths = [make_folder2(g, g.split("_")[0], pat) for g in liss]
        os.makedirs("temp", exist_ok = True)
        for gid, p in zip(liss,paths):
            ass = gid.split("_")[0]
            pp = p.split("data_submit/")[1]
            pp =  pp.replace(ass + "/" , ass + "/" + gid + "/").replace(".tar.gz", ".fna.gz")
            call("tar -C temp --strip-components=3 -vxzf  {tar} {pp} -C temp/  > /dev/null 2> /dev/null".format(tar = p, pp = pp), shell=True)
            call("tar -C temp --strip-components=3 -vxzf  {tar} {pp} -C temp/  > /dev/null 2> /dev/null".format(tar = p, pp = pp).replace(".fna.gz", ".gff.gz"), shell=True)
        checkm_dict = { k : stratfresh.loc[k, "completeness"]  for k in liss}
    for f in os.listdir("temp"):
        call("unpigz temp/" + f, shell=True)

    if sett == "gtdb":
        call("ls temp/ | cut -f1-2 -d. | parallel -j24 /home/moritz/miniconda3/envs/motupan_paper/bin/prokka  --norrna --notrna --noanno --outdir temp/retemp/{} --prefix {} --locustag {} --cpus 1 temp/{}.fna >> /dev/null  2>&1", shell = True)
        call("mv temp/retemp/*/*.gff temp/", shell=True)


    with open("temp_gff.list", "w") as handle:
        handle.writelines([c[:-4] + "\t" + "temp/" + c + "\n" for c in os.listdir("temp") if c.endswith(".gff")])
    print("running ppanggolin")

    call("""
    ppanggolin workflow --anno temp_gff.list  -o temp/ --basename temp --tmpdir temp/ -c 20 -f -K 3 > /dev/null 2> /dev/null
    python ../../mOTUlizer/bin/mOTUconvert.py --in_type ppanggolin temp/temp.h5 > temp/ppanggolin.gid2cog 2> /dev/null
    """, shell = True)

    print("running roary")
    call("""
    roary -p 22 -o temp/roary.txt  -cd 1 -v temp/*.gff > /dev/null 2> /dev/null
    mv accessory* blast_identity_frequency.Rtab  core_accessory* gene_presence_absence.* number_of_* summary_statistics.txt temp
    python ../../mOTUlizer/bin/mOTUconvert.py --in_type roary temp/roary.txt > temp/roary.gid2cog  2> /dev/null

    """,
    shell = True)

    print("running mOTUpan with ppanggolin")
    if os.stat("temp/ppanggolin.gid2cog").st_size > 0 :
        with open("temp/ppanggolin.gid2cog") as handle:
            ppan_cogs = { k : set(v) for k,v in json.load(handle).items()}
        motu = mOTU( name =  "temp" , faas = ppan_cogs, cog_dict = ppan_cogs, checkm_dict = checkm_dict, max_it = 20, threads = 20, precluster = False, method = "default")
        ppan_motupan_core = motu.core
        motu2=motu
        print("running mOTUpan with roary")

        with open("temp/roary.gid2cog") as handle:
            roary_cogs = { k : set(v) for k,v in json.load(handle).items()}
        motu = mOTU( name =  "temp" , faas = roary_cogs, cog_dict = roary_cogs, checkm_dict = checkm_dict, max_it = 20, threads = 20, precluster = False, method = "default")
        roary_motupan_core = motu.core
        roary_core = [k for k,v in motu.cogCounts.items() if v > 0.95*len(roary_cogs)]
        roary_shell = [k for k,v in motu.cogCounts.items() if v > 0.15*len(roary_cogs) and k not in roary_core]

    #    presabs = pandas.DataFrame().from_dict({k : {vv : 1.0 for vv in v} for k,v in  ppan_cogs.items()}, orient="index").fillna(0)
    #    ppangg2genome2gene = {k : {genome : [zz for zz in v if zz.startswith(genome)] for genome in set([vv.split("_CDS")[0] for vv in v])}  for k,v in ppang2gene.items()}
        handle = h5py.File("temp/temp.h5", "r")
        ppan_ppanggolin_core = {a[0].decode() for a in tqdm(handle['geneFamiliesInfo']) if a[1].decode() == "P"}
        ppan_ppanggolin_shell = {a[0].decode() for a in tqdm(handle['geneFamiliesInfo']) if a[1].decode().startswith("S")}

    #    core_mat = pandas.DataFrame.from_dict({g : { 'motupan_core' : 1 if g in ppan_motupan_core else 0, 'ppangolin_persistent' : 1 if g in ppan_ppanggolin_core else 0} for g in presabs.index}, orient = "index")


        shutil.rmtree("temp/")
        return {'motupan' : ppan_motupan_core,
                'motupan_roary' : roary_motupan_core,
                'roary_core' : roary_core,
                'roary_shell' : roary_shell,
                'new_completeness_ppan' : [g.new_completness for g in motu2],
                'ppanggolin_shell' : ppan_ppanggolin_shell,
                'ppanggolin_core' : ppan_ppanggolin_core,
                'new_completeness_roary' : [g.new_completness for g in motu],
                'roary_cogs' : roary_cogs,
                'ppan_cogs' : ppan_cogs,
                'gids' : liss
                }
    else :
        return "NA"

def process_species_cores():
    cores = {}
    cores_stats = {}

    for k,v in tqdm([(k,v) for k,v in sfdb_ids.items() if k == k and k not in cores  and k not in [ "mOTU_0110", 'mOTU_0139']]):
        print(k)
        cores[k] = process_species(v,k, sett ="stratfreshdb", subset = 50 if len(v) > 50 else None)

    for k,v in tqdm([(k,v) for k,v in gids.items() if k not in cores]):
        print(k)
        cores[k] = process_species(v,k, sett ="gtdb", subset = 50 if len(v) > 50 else None)

    core_stats = {}
    for k,v in cores.items():
        if v != "NA":
            type = "stratfreshdb" if k in sfdb_ids else "gtdb"
            tliss = sfdb_ids[k] if type =="stratfreshdb" else gids[k]
            scaffold_count = stratfresh.loc[tliss].nb_contigs if type =="stratfreshdb" else r95.loc[tliss].scaffold_count
            new_comps_roary = {gid.replace("RS_","").replace("GB_","") : len(v['roary_cogs'][gid.replace("RS_","").replace("GB_","")].intersection(v['motupan_roary']))/len(v['motupan_roary'])  for gid in v['gids']}
            new_comps_ppan = {gid.replace("RS_","").replace("GB_","") : len(v['ppan_cogs'][gid.replace("RS_","").replace("GB_","")].intersection(v['motupan']))/len(v['motupan'])  for gid in v['gids']}
#            goods = set(goods)
            core_stats[k] = {
            'taxo' : k if type == "gtdb" else "tbd", #stratfresh_motus.loc[k].consensus_tax,
            'motupan_w_ppan' : len(v['motupan']),
            'motupan_w_roary' : len(v['motupan_roary']),
            'ppanggolin_persist' : len(v['ppanggolin_core']),
            'ppanggolin_shell' : len(v['ppanggolin_shell']),
            'roary_core' : len(v['roary_core']),
            'roary_shell' : len(v['roary_shell']),
            'nb_genomes' : len(v['gids']),
            'mean_completeness_ppan' : mean(v['new_completeness_ppan']),
            'mean_completeness_roary' : mean(v['new_completeness_roary']),
            'mean_est_ppan_cogs' : mean([ len(v['ppan_cogs'][k])/c  for k,c in new_comps_roary.items() if c > 0.4] ),
            'mean_est_roary_cogs' :  mean([ len(v['roary_cogs'][k])/c  for k,c in new_comps_ppan.items() if c > 0.4] ),
            'mean_scaff_count' : mean(scaffold_count),
            'type' : type,
            'est_size' :  r95.loc[gtdb2rep[k]].est_size if type == "gtdb" else  stratfresh_motus.loc[k].est_size
            'genoms_ids' : ";".joinb(v['gids '])
            }



    pandas.DataFrame.from_dict(core_stats, orient = "index").to_csv("analyses/species_stats.csv", index_label = "species")

    paired_out = ["taxo,core_of_goods,core_of_moderate,tool,nb_genomes\n"] + ["{tax},{good_core},{bad_core},{tool},{nb}\n".format(
        tax=k,
        good_core = core_stats[k + "_goods"]['motupan'],
        bad_core = core_stats[k + "_bads"]['motupan'],
        tool = "mOTUpan",
        nb = core_stats[k + "_bads"]['nb_genomes'])
    for k in split_gids]+["{tax},{good_core},{bad_core},{tool},{nb}\n".format(
     tax=k,
     good_core = core_stats[k + "_goods"]['ppanggolin'],
     bad_core = core_stats[k + "_bads"]['ppanggolin'],
     tool = "PPanGGOLiN",
     nb = core_stats[k + "_bads"]['nb_genomes'])
    for k in split_gids]
    with open("analyses/ppanggolin_paired_stats.csv", "w") as handle:
        handle.writelines(paired_out)


def setup_data():
    os.makedirs(pjoin(folder, "nucleotides", "fnas"), exist_ok = True)
    os.makedirs(pjoin(folder, "proteins", "faas"), exist_ok = True)
    os.makedirs(pjoin(folder, "temp"), exist_ok = True)

    Entrez.email = username
    query = " OR ".join(ids)
    query = '"Prochlorococcus"[Organism]'
    ass_ids = Entrez.read(Entrez.esearch(db="assembly", term = query, retmax=1000000))['IdList']

    def get_assemblies(ass_id):
        ass_info = Entrez.read(Entrez.esummary(db="assembly", id = ass_id))
        genbank = ass_info['DocumentSummarySet']['DocumentSummary'][0]['FtpPath_GenBank']
        refseq = ass_info['DocumentSummarySet']['DocumentSummary'][0]['FtpPath_RefSeq']

        ftp_fold = genbank if refseq == "" else refseq
        ftp_file = "{fold}/{ID}_genomic.gbff.gz".format(fold = ftp_fold, ID = ftp_fold.split("/")[-1])
        call("wget {path} 2> /dev/null".format(path = ftp_file), shell = True)
        shutil.move(ftp_fold.split("/")[-1] + "_genomic.gbff.gz", "nucleotides/ncbi_gbffs")

    for ass_id in tqdm(ass_ids):
        get_assemblies(ass_id)

    genomes = dict()
    for f in tqdm(os.listdir("nucleotides/ncbi_gbffs")):
        with gzip.open("nucleotides/ncbi_gbffs" + f, "rt") as handle:
            genomes["_".join(f.split("_")[0:2])] = [s for s in SeqIO.parse(handle, "genbank")]

    def make_rec(feat, genome, seq):
        entry = ">{locus}:{genome}{ID} {product}\n{seq}\n"
        if 'translation' in feat.qualifiers:
            params = {
                'locus' : feat.qualifiers['locus_tag'][0],
                'genome' : genome,
                'ID' : (":" + feat.qualifiers['protein_id'][0]) if 'protein_id' in feat.qualifiers else "",
                'product' : feat.qualifiers['product'][0],
                'seq' : re.sub("(.{64})", "\\1\n", feat.qualifiers['translation'][0], 0, re.DOTALL)
            }
        else :
            seq = feat.extract(seq).translate(table = feat.qualifiers['transl_table'][0])
            params = {
                'locus' : feat.qualifiers['locus_tag'][0],
                'genome' : genome,
                'ID' : (":" + feat.qualifiers['protein_id'][0]) if 'protein_id' in feat.qualifiers else "",
                'product' : feat.qualifiers['product'][0],
                'seq' : re.sub("(.{64})", "\\1\n", str(seq.seq), 0, re.DOTALL)
            }
        return entry.format(**params)

    for k, v in tqdm(genomes.items()):
        SeqIO.write(v, pjoin(folder, "nucleotides", "fnas", k + ".fna"), "fasta")
        aas = [make_rec(s, k,vv) for vv in v for s in vv.features if s.type == "CDS"]
        if len(aas) > 0 :
            with open(pjoin(folder, "proteins", "faas", k + ".faa"), "w") as handle :
                handle.writelines(aas)

    full_of_Ns = "rm nucleotides/fnas/GCF_000291845.1.fna nucleotides/fnas/GCF_000291925.1.fna"

    def compute_fastamd5(file):
        with open(file) as handle:
            lines = "".join([l.strip() if not l.startswith(">") else ">" for l in handle ]).split(">")
        lines = sorted(lines)
        md5s = [md5(l.upper().encode('utf-8')).hexdigest() for l in lines]
        full_md5 = md5("".join(md5s).encode("utf-8")).hexdigest()

        return {'full_md5' : full_md5, "entry_md5s" : md5s}

    md5s_entrez = {f[:-4]: compute_fastamd5('nucleotides/fnas/' + f) for f in tqdm(os.listdir('nucleotides/fnas/'))}
    md5s_gorg = {f[:-6]: compute_fastamd5('nucleotides/gorgs/' + f) for f in tqdm(os.listdir('nucleotides/gorgs/'))}
    common_md5s = {md5['full_md5'] for md5 in md5s_entrez.values()}.intersection({md5['full_md5'] for md5 in md5s_gorg.values()})

    good_gorgs = [k for k,v in md5s_gorg.items() if v['full_md5'] not in common_md5s]
    for f in good_gorgs:
        shutil.copy('nucleotides/gorgs/' + f + ".fasta", 'nucleotides/fnas/GORG_' + f.replace("_contigs",".1.fna"))

    command = 'prokka --force --cpus 20 --genus Prochlorococcus --genus Prochlorococcus --locustag {id} --prefix {id} --outdir nucleotides/prokkas/  nucleotides/fnas/{id}.fna > /dev/null '

    for f in os.listdir('nucleotides/fnas/'):
        if f.startswith("GORG_"):
            rs_id = f[:-4]
            print(rs_id)
            call(command.format(id = rs_id), shell = True)

    "checkm taxonomy_wf -x fna -t20 genus Prochlorococcus nucleotides/fnas/  checkm   > static_data/checkm.txt"



    os.rmtree("static_data")

faas = { g[:-4] : "nucleotides/prokkas/" + g for g in  os.listdir("nucleotides/prokkas/") if g.endswith(".faa")}
checkm = parse_checkm("static_data/checkm.txt")

with open("static_data/Prochlos_mOTUs.json") as handle:
    mOTUs = json.load(handle)
mOTUS = {k : [vv['name'] for vv in v['genomes']] for k, v in mOTUs.items()}
genome2motu = {g : k for k,v in mOTUS.items() for g in v}

with open("static_data/gtdbtk/gtdbtk.bac120.summary.tsv") as handle:
    taxonomy = {l.split("\t")[0][1:] : l.split("\t")[1] for l in handle}
consensus = {k : [taxonomy.get(vv) for vv in v] for k,v in mOTUS.items()}
consensus = {k : [(s, v.count(s)) for s in set(v)] for k,v in consensus.items()}
consensus = {k: max(v, key = lambda x : x[1])[0] for k,v in consensus.items()}
blacklist = ['GCF_000634395.1']


for v in blacklist:
    del checkm[v]

full_genomes = [g for g in faas if g in checkm and checkm[g]['Completeness'] > 95]
good_genomes = [g for g in faas if g in checkm and checkm[g]['Completeness'] > 70]
decent_genomes = [g for g in faas if g in checkm and checkm[g]['Completeness'] > 40]
useable_genomes = [g for g in faas if g in checkm and checkm[g]['Completeness'] > 40]

p_As = [g for g in mOTUS['Prochlos_mOTU_002'] if g not in blacklist]
full_p_As = [k for k in p_As if k in full_genomes]

with open("nucleotides/gff_list", "w") as handle:
    handle.writelines([c + "\tnucleotides/prokkas/" + c + ".gff\n" for c in p_As])

with open("nucleotides/gff_list_fulls", "w") as handle:
    handle.writelines([c + "\tnucleotides/prokkas/" + c + ".gff\n" for c in full_p_As])


def running_tools():
    run("""
    cat nucleotides/prokkas/*.faa > nucleotides/all_faas.faa
    emapper.py --cpu 20 -m diamond --output nucleotides/all_faas -d bact -i nucleotides/all_faas.faa

#    ppanggolin annotate --anno nucleotides/gff_list  -o static_data/ --basename ppanggolin_clusters --use_pseudo --tmpdir . -c 20 -f
#    ppanggolin cluster -p static_data/ppanggolin_clusters.h5 -c 20 --tmpdir .
#    ppanggolin graph -p static_data/ppanggolin_clusters.h5 -c 20
#    ppanggolin partition -K3 -f  -p static_data/ppanggolin_clusters.h5 --cpu 20
#    ppanggolin rarefaction -c 20 --tmpdir . -K3 -f  --min 15 --depth 1 --max 1054 -o other_soft/ppanggolin/rarefaction -p static_data/ppanggolin_clusters.h5

#    mOTUconvert.py --in_type ppanggolin static_data/ppanggolin_clusters.h5 > static_data/ppanggolin.gid2cog

    ppanggolin annotate --anno nucleotides/gff_list  -o static_data/ --basename ppanggolin_clusters_species --use_pseudo --tmpdir . -c 20 -f
    ppanggolin cluster -p static_data/ppanggolin_clusters_species.h5 -c 20 --tmpdir .
    ppanggolin graph -p static_data/ppanggolin_clusters_species.h5 -c 20
    ppanggolin partition -K3 -f  -p static_data/ppanggolin_clusters_species.h5 --cpu 20
    ppanggolin rarefaction -c 20 --tmpdir . -K3 -f  --min 15 --depth 20 --max 10000 -o other_soft/ppanggolin/rarefaction_species -p static_data/ppanggolin_clusters_species.h5

    python ../../mOTUlizer/bin/mOTUconvert.py --in_type ppanggolin static_data/ppanggolin_clusters_species.h5 > static_data/ppanggolin_species.gid2cog

    ppanggolin annotate --anno nucleotides/gff_list_fulls  -o static_data/ --basename ppanggolin_clusters_goods --use_pseudo --tmpdir . -c 20 -f
    ppanggolin cluster -p static_data/ppanggolin_clusters_goods.h5 -c 20 --tmpdir .
    ppanggolin graph -p static_data/ppanggolin_clusters_goods.h5 -c 20
    ppanggolin partition -K3 -f  -p static_data/ppanggolin_clusters_goods.h5 --cpu 20

    python ../../mOTUlizer/bin/mOTUconvert.py --in_type ppanggolin static_data/ppanggolin_clusters_goods.h5 > static_data/ppanggolin_goods.gid2cog

    mkdir other_soft/roary/s__Prochlorococcus_A
    roary -p 22 -o other_soft/roary/s__Prochlorococcus_A_clusters.txt  -cd 1 -v `cat nucleotides/gff_list  | cut -f2`
    cp accessory* blast_identity_frequency.Rtab  core_accessory* gene_presence_absence.* number_of_* summary_statistics.txt other_soft/roary/s__Prochlorococcus_A
    python ../../mOTUlizer/bin/mOTUconvert.py --in_type roary other_soft/roary/s__Prochlorococcus_A_clusters.txt > static_data/roary_s__Prochlorococcus_A.gid2cog


    mkdir other_soft/roary/goods
    roary -p 20 -o other_soft/roary/goods_clusters.txt  -cd 1 -v `cat nucleotides/gff_list_fulls  | cut -f2`
    mv accessory* blast_identity_frequency.Rtab  core_accessory* gene_presence_absence.* number_of_* summary_statistics.txt other_soft/roary/goods
    mOTUconvert.py --in_type roary other_soft/roary/goods_clusters.txt > static_data/roary_goods.gid2cog

    """, shell=True)

def get_genome_stats(g):
    with open("nucleotides/prokkas/" + g + ".fna") as handle:
        length = 0
        nb_contig = 0
        for l in handle:
            if l.startswith(">") :
                nb_contig +=1
            else :
                length += len(l.strip())
    return {'genome_len' : length, 'nb_contigs' : nb_contig}
genome2stats = {g: get_genome_stats(g) for g in tqdm(genome2motu)}

def run_motupan(genomes, gid2cog, name = "test" , k=15):
    if k and k < len(genomes):
        kks = choices(genomes, k = k)
    else :
        kks = genomes
    k_dups = {k : 0 for k in set(kks) if kks.count(k) >1}
    for i,g in enumerate(kks):
        if g in k_dups:
            if k_dups[g] != 0:
                kks[i] = g +"#"+str(k_dups[g])
            k_dups[g] +=1
    faas_loc = {g : faas.get(g,faas[g.split("#")[0]])  for g in kks}
    cog_dict = {k : gid2cog.get(k,gid2cog[k.split("#")[0]])  for  k in kks}
    checkm_loc = {g : checkm.get(g,checkm[g.split("#")[0]])['Completeness']  for g in kks}
    motu = None
    motu = mOTU( name =  name , faas = faas_loc, cog_dict = cog_dict, checkm_dict = checkm_loc, max_it = 20, threads = 20, precluster = True, method = "default")
    stats = motu.get_stats()
#    roc = motu.roc_values()
    eff_genomes = sum([v for v in checkm_loc.values()])/100
    new_eff = sum([g.new_completness for g in motu])/100
    out = { 'nb_genomes' : k, 'core_len' : len(stats['test']['core']), 'aux_len' : len(stats['test']['aux_genome']), "checkm_eff" : eff_genomes, "motu_eff" : new_eff, 'mean_new_complete' : mean([g.new_completness for g in motu]) }
#    out.update(roc)
    return out

def get_data(g):
    fna = [s.seq for s in SeqIO.parse("nucleotides/prokkas/" + g + ".fna", "fasta")]
    faa = [s.seq for s in SeqIO.parse("nucleotides/prokkas/" + g + ".faa", "fasta")]
    completeness = checkm[g]['Completeness']
    contamination = checkm[g]['Contamination']
    strain_hete = checkm[g]['Strain heterogeneity']
    genome_len = sum([len(f) for f in fna])
    nb_contigs = len(fna)
    nb_cds = len([f for f in faa])
    if g.startswith("GORG_"):
        database = "GORG"
    elif g.startswith('GCF_'):
        database = "RefSeq"
    else :
        database = "GenBank"

    return (g, { 'completeness' : completeness,
                'contamination' : contamination,
                'strain_heterogeneity' : strain_hete,
                'nb_bases' : genome_len,
                'nb_scaffolds' : nb_contigs,
                'nb_CDSs' : nb_cds,
                'database' : database
    })

def make_file_stats():
    dd = dict([get_data(g) for g in tqdm(p_As)])
    dd = pandas.DataFrame.from_dict(dd, orient="index")

def ppanggolin_vs_motulizer():

    with open("static_data/ppanggolin_species.gid2cog") as handle:
        gid2cog_species = json.load(handle)
    gid2cog_species = {k : set(v) for k,v in gid2cog_species.items()}
    j = list(range(15,len(gid2cog_species), 1))*5
    def motupan_species_test_w_ppanggolin_cogs(i):
        return run_motupan(p_As,k=j[i], gid2cog = gid2cog_species)

    motupan_ppanggolin_species = mOTU( name =  "motupan_ppanggolin_species" ,
            faas = {g : v for g,v in faas.items() if g in gid2cog_species} ,
            cog_dict = gid2cog_species,
            checkm_dict = {g : checkm[g]['Completeness'] for g in gid2cog_species}, max_it = 20, method = "default")


    pool = Pool(processes=20)
    motupan_species_rarefact_w_ppanggolin_cogs = pool.map(motupan_species_test_w_ppanggolin_cogs, list(range(len(j))))
    with open("analyses/motupan_species_rarefact_w_ppanggolin_cogs.tsv", "w") as handle:
        head = list(motupan_species_rarefact_w_ppanggolin_cogs[0].keys())
        ppangg2genome2genehandle.writelines(["\t".join(head) + "\n"])
        handle.writelines(["\t".join([str(l[k]) for k in head]) + "\n"  for l in motupan_species_rarefact_w_ppanggolin_cogs])

    handle = h5py.File("static_data/ppanggolin_clusters_species.h5", "r")
    gene2ppangg = {a.decode() : b.decode() for a,b in  tqdm(handle['geneFamilies'])}
    ppang2gene = {s : [] for s in set(gene2ppangg.values())}
    for a,b in gene2ppangg.items():
        ppang2gene[b] += [a]

    ppangg2genome2gene = {k : {genome : [zz for zz in v if zz.startswith(genome)] for genome in set([vv.split("_CDS")[0] for vv in v])}  for k,v in ppang2gene.items()}

    ppanggolin_sets = {a[0].decode() : {'ppanggolin' : 'persistent' if a[1].decode() == "P" else 'shell' if a[1].decode() == "S" else 'cloud'}  for a in tqdm(handle['geneFamiliesInfo'])}
    for k, v in ppanggolin_sets.items():
        ppanggolin_sets[k]['motupan'] = 'core' if k in motupan_ppanggolin_species.core else 'accessory'


    ppanggolin_matrix = {gg : dict({k : 0 for k in gid2cog_species}) for g in gid2cog_species.values() for gg in g}
    for k, gs in gid2cog_species.items():
        for gg in gs:
            ppanggolin_matrix[gg][k] = 1

    pandas.DataFrame.from_dict(ppanggolin_sets, orient = "index").to_csv("analyses/ppanggolin_matrix_species_cogs.csv", index_label = "COG")
    pandas.DataFrame.from_dict(ppanggolin_matrix, orient = "index").to_csv("analyses/ppanggolin_matrix_species.csv", index_label = "cog")
    pandas.DataFrame.from_dict({g.name : {'motupan_completeness' : g.new_completness, 'checkm_completeness' : g.checkm_complet, **genome2stats[g.name]} for g in motupan_ppanggolin_species}, orient="index").to_csv("analyses/ppanggolin_matrix_species_genomes.csv", index_label = "genome")



def pange_dict2roary_classes(gid2cog, mean_complete = 100):
    core_cutoff = 0.99*mean_complete/100
    softcore_cutoff = 0.95*mean_complete/100
    shell_cutoff = 0.15*mean_complete/100
    cloud_cutoff = 0

    nb_genomes = len(gid2cog)
    all_cog_counts = {vv : 0 for v in gid2cog.values() for vv in v}

    for v in gid2cog.values():
        for vv in v:
            all_cog_counts[vv] += 1

    core = {k for k,v in all_cog_counts.items() if v/nb_genomes > core_cutoff}
    for k in core:
        del all_cog_counts[k]

    softcore = {k for k,v in all_cog_counts.items() if v/nb_genomes > softcore_cutoff}
    for k in softcore:
        del all_cog_counts[k]

    shell = {k for k,v in all_cog_counts.items() if v/nb_genomes > shell_cutoff}
    for k in shell:
        del all_cog_counts[k]

    cloud = set(all_cog_counts.keys())
    return {'core' : len(core), 'softcore' : len(softcore), 'shell' : len(shell), 'cloud' : len(cloud) }

def roary_vs_motupan():
    with open("static_data/roary_goods.gid2cog") as handle:
        roary_gi2cog = {k : set(v) for k, v in json.load(handle).items()}
        reps=20
        boots = []
        for i in tqdm(range(3, len(roary_gi2cog))):
            for j in range(reps):
                sub_gid2cog = {k : roary_gi2cog[k] for k in choices(list(roary_gi2cog), k=i)}
                est_complete = mean([checkm[k]['Completeness'] for k in sub_gid2cog])
                tt = pange_dict2roary_classes(sub_gid2cog)
                tt['nb_org'] = i
                tt['rep'] = j
                tt['method'] = "strict"
                tt['est_checkm_complete'] = est_complete
                motupan = run_motupan(list(sub_gid2cog.keys()), k=i, gid2cog = sub_gid2cog)

                tt['motupan_est_checkm'] = motupan['mean_new_complete']
                tt['motupan_core'] = motupan['core_len']
                tt['motupan_cloud'] = motupan['aux_len']

                boots += [tt]

        head = list(boots[0].keys())
        with open("analyses/roary_rarefaction.csv", "w") as handle:
            handle.writelines([",".join(head) + "\n"] + [ ",".join([str(dd[k]) for k in head]) + '\n' for dd in boots])

    with open("static_data/ppanggolin_goods.gid2cog") as handle:
        ppanggolin_gi2cog = {k : set(v) for k, v in json.load(handle).items()}
        boots = []
        for i in tqdm(range(3, len(ppanggolin_gi2cog))):
            for j in range(reps):
                sub_gid2cog = {k : ppanggolin_gi2cog[k] for k in choices(list(ppanggolin_gi2cog), k=i)}
                est_complete = mean([checkm[k]['Completeness'] for k in sub_gid2cog])
                tt = pange_dict2roary_classes(sub_gid2cog)
                tt['nb_org'] = i
                tt['rep'] = j
                tt['method'] = "strict"
                tt['est_checkm_complete'] = est_complete
                motupan = run_motupan(list(sub_gid2cog.keys()),k=i, gid2cog = sub_gid2cog)

                tt['motupan_est_checkm'] = motupan['mean_new_complete']
                tt['motupan_core'] = motupan['core_len']
                tt['motupan_cloud'] = motupan['aux_len']

                boots += [tt]

        head = list(boots[0].keys())
        with open("analyses/ppanggolin_rarefaction.csv", "w") as handle:
            handle.writelines([",".join(head) + "\n"] + [ ",".join([str(dd[k]) for k in head]) + '\n' for dd in boots])



def processing_goods():
    with open("static_data/roary_goods.gid2cog") as handle:
        roary = {k : set(v) for k, v in json.load(handle).items()}
    with open("static_data/ppanggolin_goods.gid2cog") as handle:
        ppanggolin = {k : set(v) for k, v in json.load(handle).items()}

    motupan_roary = mOTU( name =  "motupan_roary" ,
            faas = {g : v for g,v in faas.items() if g in roary} ,
            cog_dict = {g : v for g,v in roary.items() if g in roary},
            checkm_dict = {g : checkm[g]['Completeness'] for g in roary}, max_it = 20, method = "default")

    motupan_ppanggolin = mOTU( name =  "motupan_ppanggolin" ,
            faas = {g : v for g,v in faas.items() if g in ppanggolin} ,
            cog_dict = {g : v for g,v in ppanggolin.items() if g in ppanggolin},
            checkm_dict = {g : checkm[g]['Completeness'] for g in ppanggolin}, max_it = 20, method = "default")

    motupan_roary.roc_values()
    motupan_ppanggolin.roc_values()
    handle = h5py.File("static_data/ppanggolin_clusters_goods.h5", "r")
    gene2ppangg = {a.decode() : b.decode() for a,b in  tqdm(handle['geneFamilies'])}
    ppang2gene = {s : [] for s in set(gene2ppangg.values())}
    for a,b in gene2ppangg.items():
        ppang2gene[b] += [a]

    ppangg2genome2gene = {k : {genome : [zz for zz in v if zz.startswith(genome)] for genome in set([vv.split("_CDS")[0] for vv in v])}  for k,v in ppang2gene.items()}

    ppanggolin_sets = {a[0].decode() : {'ppanggolin' : 'persistent' if a[1].decode() == "P" else 'shell' if a[1].decode() == "S" else 'cloud'}  for a in tqdm(handle['geneFamiliesInfo'])}
    for k, v in ppanggolin_sets.items():
        ppanggolin_sets[k]['motupan'] = 'core' if k in motupan_ppanggolin.core else 'accessory'



    ppanggolin_matrix = {gg : dict({k : 0 for k in ppanggolin}) for g in ppanggolin.values() for gg in g}
    for k, gs in ppanggolin.items():
        for gg in gs:
            ppanggolin_matrix[gg][k] = 1

    pandas.DataFrame.from_dict(ppanggolin_matrix, orient = "index").to_csv("analyses/ppanggolin_matrix_species.csv", index_label = "cog")
    pandas.DataFrame.from_dict({g.name : {'motupan_completeness' : g.new_completness, 'checkm_completeness' : g.checkm_complet, **genome2stats[g.name]} for g in motupan_ppanggolin}, orient="index").to_csv("analyses/ppanggolin_matrix_genomes.csv", index_label = "genome")

    roary_matrix = {gg : dict({k : 0 for k in roary}) for g in roary.values() for gg in g}



import abc
from tqdm import tqdm
import sys
import h5py
import hdf5plugin

class Parser(metaclass=abc.ABCMeta):

    def __init__(self, **kwargs):
        if 'gene_id2genome' in kwargs and gene_id2genome:
            if type(gene_id2genome) == dict:
                self.gene_id2genome = gene_id2genome
            else :
                print("TODO")
                sys.exit(0)
        else :
            self.gene_id2genome = None

    @abc.abstractmethod
    def convert(self, infile, outfile = None, **kwargs):
        pass

class EmapperParse(Parser):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        try :
            from ete3 import NCBITaxa
        except :
            print("""
            To parse eggNOGs properly I need the ete3 python package,
            you could for example install it with pip as in :
                pip install ete3

            or, if conda is more your cupa tea, you can do :
                conda install -c conda-forge -c etetoolkit ete3=3.1.2

            or whichever other way you like. I don't judge.
            """, file = sys.stderr)
            sys.exit(0)

    def convert(self, infile, count = False):
        from ete3 import NCBITaxa
        tax_db = NCBITaxa()

        with open(infile) as handle:
            header = None
            counter = 0
            while not header and counter < 15:
                head = handle.readline().rstrip().split("\t")
                if head[0] == "#query_name" :
                    header = head
                counter += 1
            if not header :
                print("\nYou sure this file is good? Like, is it the '.emapper.annotations' you got from running eggnoggmapper?\n")
                sys.exit(0)

            idx = [i for i,v in enumerate(header) if v == "eggNOG OGs"][0]
            print("Loading eggNOGs from file.", file = sys.stderr)
            gene_id2eggs = {l.split("\t")[0] : l.split("\t")[idx] for l in tqdm(handle.readlines()) if not l.startswith("#")}


        print("Parsing taxonomies, and simplifying to deepest eggNOG.", file = sys.stderr)
        taxos = {vv.split("@")[1] for v in gene_id2eggs.values() for vv in v.split(",")}
        tax2level = {k : len(v) for k,v in tax_db.get_lineage_translator(list(taxos)).items()}
        lowest_level = lambda x : tax2level.get(int(x[1]),1000)
        gene_id2deepest_egg = {k:  min([vv.split('@') for vv in v.split(",")], key = lowest_level)[0] for k,v in tqdm(gene_id2eggs.items())}

        print("Stratify it to genome", file = sys.stderr)
        if not self.gene_id2genome:
            self.gene_id2genome = {k : "_".join(k.split("_")[:-1]) for k in gene_id2deepest_egg.keys()}

        genome2nog = {k : [] for k in set(self.gene_id2genome.values())}
        for k,v in gene_id2deepest_egg.items():
            genome2nog[self.gene_id2genome[k]] += [v]
        if count:
            return {k : {vv : v.count(vv) for vv in set(v)} for k,v in genome2nog.items()}
        else :
            return {k : list(set(v)) for k,v in genome2nog.items()}



class PPanGGolinParse(Parser):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        try :
            import h5py
            import hdf5plugin
        except :
            print("""
            To parse PPanGGolin's h5 file properly I need the h5py and hdf5plugin
            python package, you could for example install it with pip as in :
                pip install h5py hdf5plugin

            or, if conda is more your cupa tea, you can do :
                conda install -c anaconda -c conda-forge ete3

            or whichever other way you like. I don't judge.
            """, file = sys.stderr)
            sys.exit(0)


    def convert(self, infile, count = False):
        import h5py
        import hdf5plugin

        print("Parse and decode gene-clusters from the h5py-file", file = sys.stderr)
        with h5py.File(infile, "r") as handle:
            gene_id2family = {a.decode() : b.decode() for a,b in tqdm(handle['geneFamilies'])}

        print("Stratify it to genome", file = sys.stderr)
        if not self.gene_id2genome:
            self.gene_id2genome = {k : "_".join(k.split("_")[:-2]) for k in gene_id2family.keys()}

        genome2family = {k : [] for k in set(self.gene_id2genome.values())}
        for k,v in gene_id2family.items():
            genome2family[self.gene_id2genome[k]] += [v]
        if count:
            return {k : {vv : v.count(vv) for vv in set(v)} for k,v in genome2family.items()}
        else :
            return {k : list(set(v)) for k,v in genome2family.items()}

class RoaryParse(Parser):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def convert(self, infile, count = False):
        print("Parse the cluster-file", file = sys.stderr)
        with open(infile, "r") as handle:
            gene_id2family = {ll : l.split(':')[0] for l in tqdm(handle) for ll in l.split(":")[1].strip().split()}

        print("Stratify it to genome", file = sys.stderr)
        if not self.gene_id2genome:
            self.gene_id2genome = {k : "_".join(k.split("_")[:-1]) for k in gene_id2family.keys()}

        genome2family = {k : [] for k in set(self.gene_id2genome.values())}
        for k,v in gene_id2family.items():
            genome2family[self.gene_id2genome[k]] += [v]
        if count:
            return {k : {vv : v.count(vv) for vv in set(v)} for k,v in genome2family.items()}
        else :
            return {k : list(set(v)) for k,v in genome2family.items()}

class MmseqsParse(Parser):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def convert(self, infile, count = False):
        print("Parse the cluster-file", file = sys.stderr)
        with open(infile, "r") as handle:
            gene_id2family = {l.strip().split('\t')[1] : l.strip().split('\t')[0] for l in tqdm(handle)}

        print("Stratify it to genome", file = sys.stderr)
        if not self.gene_id2genome:
            self.gene_id2genome = {k : "_".join(k.split("_")[:-1]) for k in gene_id2family.keys()}

        genome2family = {k : [] for k in set(self.gene_id2genome.values())}
        for k,v in gene_id2family.items():
            genome2family[self.gene_id2genome[k]] += [v]
        if count:
            return {k : {vv : v.count(vv) for vv in set(v)} for k,v in genome2family.items()}
        else :
            return {k : list(set(v)) for k,v in genome2family.items()}



from mOTUlizer.classes.MetaBin import MetaBin
from mOTUlizer.classes.COGs import *
import subprocess
import tempfile
import os
from mOTUlizer.config import *
from random import shuffle, choice, choices
from math import log10
import sys
import json


mean = lambda x : sum(x)/len(x)

class mOTU:
    def __len__(self):
        return len(self.members)

    def __repr__(self):
        return "< {tax} mOTU {name}, of {len} members >".format(name = self.name, len = len(self), tax =  None ) #self.consensus_tax()[0].split(";")[-1])

    def __init__(self, **kwargs):
        self.quiet = kwargs['quiet'] if "quiet" in kwargs else False
        self.likelies = None
        self.mock = []
        if "cog_dict" in kwargs:
            self.__for_mOTUpan(**kwargs)

        if "bins" in kwargs:
            self.__from_bins(**kwargs)


    def __for_mOTUpan(self, name, faas, cog_dict, checkm_dict, threads = 4, precluster = False, max_it = 20, method = None, quiet=False):
        self.name = name
        self.faas = faas
        if not quiet:
            print("Creating mOTU for mOTUpan", file = sys.stderr)
        if  not cog_dict :
            tt = compute_COGs(self.faas, name = name, precluster = precluster, threads = threads)
            self.cog_dict = tt['genome2cogs']
            self.aa2cog = tt['aa2cog']
        else :
            self.cog_dict = cog_dict
            self.aa2cog = {}

        if checkm_dict == "length_seed" :
            max_len = max([len(cogs) for cogs in self.cog_dict.values()])
            checkm_dict = {}
            for f in self.cog_dict:
                checkm_dict[f] = 100*len(self.cog_dict[f])/max_len

        self.members = [MetaBin(bin_name, cogs = self.cog_dict[bin_name], faas = self.faas.get(bin_name), fnas = None, complet = checkm_dict.get(bin_name)) for bin_name in self.cog_dict.keys()]
        self.cogCounts = {c : 0 for c in set.union(set([cog for mag in self.members for cog in mag.cogs]))}
        for mag in self.members:
            for cog in mag.cogs:
                    self.cogCounts[cog] += 1

        self.core = {cog for cog, counts in self.cogCounts.items() if (100*counts/len(self)) > mean(checkm_dict.values())}

        self.method = method
        if self.method :
            self.likelies = self.__core_likelyhood(max_it = max_it)


    def __getitem__(self, i):
        return self.members[i]


    def roc_values(self, boots):
        if boots > 0:
            from mOTUlizer.classes.MockData import MockmOTU
            mean = lambda data: float(sum(data)/len(data))
            variance 	= lambda data, avg: sum([x**2 for x in [i-avg for i in data]])/float(len(data))
            std_dev = lambda data: variance(data, mean(data))**0.5

            while len(self.mock) < boots:
                print("Running bootstrap {}/{}".format(len(self.mock)+1, boots), file = sys.stderr)
                completnesses = {"Genome_{}".format(i) : c.new_completness for i,c in enumerate(self)}

                accessory = sorted([v for k,v in self.cogCounts.items() if k not in self.core])
                missing = int(sum(accessory)*(1-mean(list(completnesses.values()))/100))
                if len(accessory) > 0:
                    addeds = choices(list(range(len(accessory))), weights = accessory, k = missing)
                    for k in addeds :
                        accessory[k] += 1

                self.mock += [MockmOTU(self.name + "_mock", len(self.core), len(self), lambda g : completnesses[g], accessory = accessory, method = self.method)]
            return { 'mean_recall' : mean([m.recall for m in self.mock]),
                     'sd_recall' : std_dev([m.recall for m in self.mock]),
                     'mean_fpr' : mean([m.fpr for m in self.mock]),
                     'sd_fpr' : std_dev([m.fpr for m in self.mock]),
                     'mean_lowest_false' : mean([m.lowest_false for m in self.mock]),
                     'sd_lowest_false' : std_dev([m.lowest_false for m in self.mock])}
        else:
            return { 'mean_recall' : "NA",
                     'sd_recall' : "NA",
                     'mean_fpr' : "NA",
                     'sd_fpr' : "NA",
                     'mean_lowest_false' : "NA",
                     'sd_lowest_false' : "NA"}


    def avg_cog_content(self):
        return sum([len(m.cogs) for m in self.members])/len(self)

    def consensus_tax(self):
        taxos = [bin.taxonomy for bin in self.members]
        temp = {}
        for i in range(0,7):
            leveled = [t[i] for t in taxos]
            counts = {l : leveled.count(l) for l in set(leveled)}
            win = max(counts.items(), key = lambda x : x[1])
            temp[i] = win
        output = ["", []]
        for i in range(7):
            if temp[i][0] == "":
                break
            output[0] += ";" + temp[i][0]
            output[1] += [temp[i][1] / len(self)]
        output[0] = output[0][1:]
        return tuple(output)

    def overlap_matrix(self):
        if not hasattr(self, 'overlap_dict'):
            self.overlap_dict = {(i,j) : len(i.overlap(j))/len(i.cogs) for i in self.members for j in self.members if i != j}
        return self.overlap_dict

    def mean_overlap(self) :
        return mean(list(self.overlap_matrix().values()))

    def fastani_matrix(self):
        if not hasattr(self, 'fastani_dict'):
            if not shutil.which('fastANI'):
                print("You need fastANI if you do not provide a file with pairwise similarities, either install it or provide pairwise similarities (see doc...)", file = sys.stderr)
                sys.exit(-1)
            cmd = "fastANI --ql {temp_file}  --rl {temp_file}  -o {temp_out} -t {threads} 2> /dev/null"
            temp_inp = tempfile.NamedTemporaryFile()
            temp_out = tempfile.NamedTemporaryFile()

            with open(temp_inp.name, "w") as handle:
                handle.writelines([t.genome + "\n" for t in self.members ])

            cmd = cmd.format(temp_file = temp_inp.name, temp_out = temp_out.name, threads = THREADS)

            subprocess.call(cmd, shell=True)

            with open(temp_out.name ) as handle:
                self.fastani_dict = {}
                for l in handle:
                    q = os.path.basename(l.split()[0][:-4])
                    s = os.path.basename(l.split()[1][:-4])
                    if q  != s:
                        value = float(l.split()[2])
                        self.fastani_dict[(q,s)] = value
        return self.fastani_dict


    def get_stats(self):
        out = {}

        out[self.name] = { "nb_genomes" : len(self),
        "core" : list(self.core) if self.core else None,
        "aux_genome" : [k for k,v in self.cogCounts.items() if k not in self.core] if self.core else None ,
        "singleton_cogs" : [k for k,v in self.cogCounts.items() if k not in self.core if v == 1] if self.core else None,
        "cogs" : None if self.cog_dict is None else {'genome' : {k : list(v) for k,v in self.cog_dict.items()}, 'aa' : self.aa2cog} if self.aa2cog else ({k : list(v) for k,v  in self.cog_dict.items()} if self.cog_dict else None),
        "mean_ANI" : self.get_mean_ani() if (hasattr(self, 'fastani_dict') or all([hasattr(g, "genome") for g in self])) else None,
        "ANIs" : [[k[0], k[1], v] for k, v in self.fastani_matrix().items()] if (hasattr(self, 'fastani_dict')  or all([hasattr(g, "genome") for g in self])) else None,
        "genomes" : [v.get_data() for v in self],
        "likelies" : self.likelies
        }
        return out

    def get_representative(self, method = "complex", max_redund = 5, min_complete = 95):
            tt = [v.get_data() for v in self]
            data = { t['name'] : t for t in tt}
            if all([v['checkm_contamin'] > max_redund for v in data.values()]):
                return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]
            data = {k : v for k,v in data.items() if v['checkm_contamin'] < max_redund}
            if any([v['checkm_complet'] > min_complete for v in data.values()])  :
                data = {k : v for k,v in data.items() if v['checkm_complet'] > min_complete}
                best_redund = min(data.items(), key = lambda x : x[1]['checkm_contamin'])[1]['checkm_contamin']
                return max( [ (k , v['checkm_complet']) for k,v in data.items() if v['checkm_contamin'] == best_redund], key = lambda x: x[1])[0]
            else:
                return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]

    # def get_representative(tt, max_redund = 5, min_complete = 95):
    #         data = { t['name'] : t for t in tt}
    #
    #         data = {k : v for k,v in data.items() if v['checkm_contamin'] < max_redund}
    #         if any([v['checkm_complet'] > min_complete for v in data.values()])  :
    #             data = {k : v for k,v in data.items() if v['checkm_complet'] > min_complete}
    #             best_redund = min(data.items(), key = lambda x : x[1]['checkm_contamin'])[1]['checkm_contamin']
    #             return min( [ (k , v['checkm_contamin']) for k,v in data.items() if v['checkm_contamin'] == best_redund], key = lambda x: x[1])[0]
    #         elif len(data) >0 :
    #             return max( [ (k , v['checkm_complet']) for k,v in data.items()], key = lambda x: x[1])[0]
    #         else :
    #             return None

    def get_mean_ani(self):
        dist_dict = self.fastani_matrix()
        dists = [dist_dict.get((a.name,b.name)) for a in self for b in self if a != b ]
        missing_edges = sum([d is None for d in dists])
        found_edges = [d is None for d in dists]

        return {'mean_ANI' : sum([d for d in dists if d])/len(found_edges) if len(found_edges) > 0 else None, 'missing_edges' : missing_edges, 'total_edges' : len(found_edges) + missing_edges}

    def __core_likelyhood(self, max_it = 20, likeli_cutof = 0 ):
        likelies = {cog : self.__core_likely(cog) for cog in self.cogCounts}
        self.core = set([c for c, v in likelies.items() if v > likeli_cutof])
        core_len = len(self.core)
        i = 1
        if not self.quiet:
            print("iteration 1 : ", core_len, "sum_abs_LLHR:" , sum([l if l > 0 else -l for l in likelies.values()]), file = sys.stderr)
        for mag in self:
            if len(self.core) > 0:
                mag.new_completness = 100*len(mag.cogs.intersection(self.core))/len(self.core)
            else :
                mag.new_completness = 0
            mag.new_completness = mag.new_completness if mag.new_completness < 99.9 else 99.9
            mag.new_completness = mag.new_completness if mag.new_completness > 0 else 0.01
        for i in range(2,max_it):
            likelies = {cog : self.__core_likely(cog, complet = "new", core_size = core_len) for cog in self.cogCounts}
            old_core = self.core
            self.core = set([c for c, v in likelies.items() if v > likeli_cutof])
            new_core_len = len(self.core)
            for mag in self:
                if len(self.core) > 0:
                    mag.new_completness = 100*len(mag.cogs.intersection(self.core))/len(self.core)
                else :
                    mag.new_completness = 0
                mag.new_completness = mag.new_completness if mag.new_completness < 99.9 else 99.9
                mag.new_completness = mag.new_completness if mag.new_completness > 0 else 0.01
            if not self.quiet:
                print("iteration",i, ": ", new_core_len, "sum_abs_LLHR:" , sum([l if l > 0 else -l for l in likelies.values()]), file = sys.stderr)
            if self.core == old_core:
               break
            else :
                core_len = new_core_len

        if not self.quiet:
            json.dump({ self.name : {"nb_mags" : len(self), "core_len" : core_len, "mean_starting_completeness" :  mean([b.checkm_complet for b in self]), "mean_new_completness" : mean([b.new_completness for b in self]), "LHR"  :  sum([l if l > 0 else -l for l in likelies.values()]), "mean_est_binsize" : mean([100*len(b.cogs)/b.new_completness for b in self])}}, sys.stderr )
            print(file = sys.stderr)
        self.iterations = i -1
        return likelies

    def __core_prob(self, cog, complet = "checkm"):
        comp = lambda mag : (mag.checkm_complet if complet =="checkm" else mag.new_completness)/100
        presence = [log10(comp(mag)) for mag in self if cog in mag.cogs]
        abscence = [log10(1 - comp(mag)) for mag in self if cog not in mag.cogs]
        return sum(presence + abscence)

    def __pange_prob(self, cog, core_size, complet = "checkm"):
#        pool_size = sum(self.cogCounts.values())
        pool_size = sum([c for k,c in  self.cogCounts.items()])
        comp = lambda mag : (mag.checkm_complet if complet =="checkm" else mag.new_completness)/100
        #presence = [1 - (1-self.cogCounts[cog]/pool_size)**(len(mag.cogs)-(core_size*comp(mag))) for mag in self if cog in mag.cogs]
        #abscence = [ (1-self.cogCounts[cog]/pool_size)**(len(mag.cogs)-(core_size*comp(mag))) for mag in self if cog not in mag.cogs]

#        mag_prob = {mag : ( 1-1/pool_size )**len(mag.cogs) for mag in self}
#        mag_prob = {mag : ( 1-1/pool_size )**(len(mag.cogs)-(core_size*comp(mag))) for mag in self}
#        mag_prob = {mag : ( 1-self.cogCounts[cog]/pool_size )**(len(mag.cogs)-(core_size*comp(mag))) for mag in self}

#        mag_prob = {mag : ( 1-self.cogCounts[cog]/pool_size )**(len(mag.cogs)-(core_size*comp(mag))) for mag in self}
        mag_prob = {mag : ( 1-self.cogCounts[cog]/pool_size)**len(mag.cogs) for mag in self}

        presence = [ log10(1 -   mag_prob[mag]) if mag_prob[mag] < 1 else MIN_PROB                for mag in self if cog in mag.cogs]
        abscence = [ log10(      mag_prob[mag]) if mag_prob[mag] > 0 else log10(1-(10**MIN_PROB)) for mag in self if cog not in mag.cogs]

        #abscence = [ 1-self.cogCounts[cog]/len(self)*comp(mag) for mag in self if cog not in mag.cogs]
        #presence = [ self.cogCounts[cog]/len(self)*comp(mag) for mag in self if cog not in mag.cogs]

        return sum(presence + abscence)

    def __core_likely(self, cog, complet = "checkm", core_size = 0):
        pange_prob = self.__pange_prob(cog, core_size, complet)
        return self.__core_prob(cog, complet) - pange_prob

    def nb_cogs(self):
        return mean([b.estimate_nb_cogs() for b in self if b.new_completness > 40 ])

    def get_pangenome_size(self, singletons = False):
        return len([k for k,v in self.cogCounts.items() if k not in self.core and v > (0 if singletons else 1)])


    def __from_bins(self, bins, name,dist_dict = None ):
        self.name = name

        self.members = bins
        self.core = None
        self.fastani_dict = dist_dict
        self.aa2cog = None
        self.likelies = None
        self.cog_dict = None

    @classmethod
    def cluster_MetaBins(cls , all_bins, dist_dict, ani_cutoff = 95, prefix = "mOTU_", mag_complete = 40, mag_contamin = 5, sub_complete = 0, sub_contamin = 100):
        import igraph

        print("seeding bin-graph", file = sys.stderr )

        all_bins = {a.name : a for a in all_bins}

        tt = [(k, v.checkm_complet, v.checkm_contamin) for k, v in all_bins.items() if v.checkm_complet > 0]

        good_mag = lambda b : all_bins[b].checkm_complet > mag_complete and all_bins[b].checkm_contamin < mag_contamin
        decent_sub = lambda b : all_bins[b].checkm_complet > sub_complete and all_bins[b].checkm_contamin < sub_contamin and not good_mag(b)
        good_pairs = [k for k,v  in dist_dict.items() if v > ani_cutoff and dist_dict.get((k[1],k[0]), 0) > ani_cutoff and good_mag(k[0]) and good_mag(k[1])]
        species_graph = igraph.Graph()
        vertexDeict = { v : i for i,v in enumerate(set([x for k in good_pairs for x in k]))}
        rev_vertexDeict = { v : i for i,v in vertexDeict.items()}
        species_graph.add_vertices(len(vertexDeict))
        species_graph.add_edges([(vertexDeict[k[0]], vertexDeict[k[1]]) for k in good_pairs])

        print("getting clusters", file = sys.stderr)

        genome_clusters = [[rev_vertexDeict[cc] for cc in c ] for c in species_graph.components(mode=igraph.STRONG)]

        mean = lambda l : sum([len(ll) for ll in l])/len(l)

        print("recruiting to graph of the", len(genome_clusters) ," mOTUs of mean length", mean(genome_clusters), file = sys.stderr)


        left_pairs = {k : v for k, v in dist_dict.items() if v > ani_cutoff and k[0] != k[1] and ((decent_sub(k[0]) and good_mag(k[1])) or (decent_sub(k[1]) and good_mag(k[0])))}
        print("looking for good_left pairs", file = sys.stderr)
#        print(left_pairs)

        subs = {l : (None,0) for ll in left_pairs.keys() for l in ll if not good_mag(l)}
#        print(subs)
        print("looking for best mOTU match", file = sys.stderr)
        for p,ani in left_pairs.items():
            if p[0] in subs and subs[p[0]][1] < ani:
                subs[p[0]] = (p[1], ani)
            if p[1] in subs and subs[p[1]][1] < ani:
                subs[p[1]] = (p[0], ani)

        genome_clusters = [set(gg) for gg in genome_clusters]

        print("append to the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters), file = sys.stderr)
        for k, v in subs.items():
            for g in genome_clusters:
                if v[0] in g :
                    g.add(k)

        genome_clusters = [list(gg) for gg in genome_clusters]

        print("processing the", len(genome_clusters) ,"mOTUs of mean length", mean(genome_clusters), file = sys.stderr)
        #print(genome_clusters)

        zeros = len(str(len(genome_clusters)))
        motus = [ mOTU(bins = [all_bins[gg] for gg in gs], name = prefix + str(i).zfill(zeros), dist_dict = {(k,l) : dist_dict[(k,l)] for k in gs for l in gs if (k,l) in dist_dict}) for i, gs in enumerate(genome_clusters)]


        return motus



from os.path import join as pjoin
import tempfile
import sys
from subprocess import Popen, PIPE, call
import os
import shutil
from mOTUlizer.config import FASTA_EXTS

class MetaBin:
    def __repr__(self) :
        return "< bin {name} with {n} cogs>".format(n = len(self.cogs) if cogs else "NA", name = self.name)

    def __init__(self, name, cogs,fnas, faas, complet = None, contamin = 0, max_complete = 99.9):
        self.name = name
        self.cogs = cogs if type(cogs) != str else set([cogs])
        self.faas = faas
        self.fnas = fnas
        self.checkm_complet = complet
        self.checkm_contamin = contamin
        if not self.checkm_complet is None:
            if self.checkm_complet > max_complete:
                self.checkm_complet =  max_complete
        self.new_completness = None

    def get_data(self):
        return { 'name' : self.name,
                 'faa-file' : self.faas,
                 'fna-file' : self.fnas,
                 'checkm_complet' : self.checkm_complet,
                 'checkm_contamin' : self.checkm_contamin,
                 'new_completness' : self.new_completness,
        }


    def overlap(self, target):
        return self.cogs.intersection(target.cogs)

    def estimate_nb_cogs(self):
        assert self.new_completness != None, "new_completness not computed, please do"
        return 100*len(self.cogs)/self.new_completness

    @classmethod
    def get_anis(cls, bins, outfile = None, method = "fastANI", block_size = 500, threads=1):
        if method == "fastANI":
            if not shutil.which('fastANI'):
                print("You need fastANI if you do not provide a file with pairwise similarities, either install it or provide pairwise similarities (see doc...)", file = sys.stderr)
                sys.exit(-1)
            fastani_file = tempfile.NamedTemporaryFile().name if outfile is None else outfile

            mags = [b.fnas for b in bins]

            mag_blocks = [mags[i:(i+block_size)] for i in list(range(0,len(mags), block_size))]

            if len(mag_blocks) > 1:
                print("You have more then {bsize} bins, so we will run fastANI in blocks, if it crashes due to memory, make smaller blocks".format(bsize = block_size), file=sys.stderr)

            with open(fastani_file, "w") as handle:
                handle.writelines(["query\tsubject\tani\tsize_q\tsize_s\n"])

            for i,bloc1 in enumerate(mag_blocks):
                b1_tfile = tempfile.NamedTemporaryFile().name

                with open(b1_tfile, "w") as handle:
                    handle.writelines([l +"\n" for l in bloc1])

                for j,bloc2 in enumerate(mag_blocks):
                        print("doing bloc {i} and {j}".format(i = i, j=j), file = sys.stderr)
                        b2_tfile = tempfile.NamedTemporaryFile().name
                        with open(b2_tfile, "w") as handle:
                            handle.writelines([l  +"\n" for l in bloc2])

                        out_tfile = tempfile.NamedTemporaryFile().name
                        call("fastANI --ql {b1} --rl {b2} -o {out} -t {threads} 2> /dev/null".format(b1 = b1_tfile, b2 = b2_tfile, out = out_tfile, threads = threads), shell = True)
                        with open(out_tfile) as handle:
                            new_dat = ["\t".join([ll for ll in l.split()]) +"\n" for l in handle.readlines()]
                        with open(fastani_file, "a") as handle:
                            handle.writelines(new_dat)

                        os.remove(out_tfile)
                        os.remove(b2_tfile)

            os.remove(b1_tfile)
            with open(fastani_file) as handle:
                handle.readline()
                out_dists = {(os.path.basename(l.split()[0]), os.path.basename(l.strip().split()[1])) : float(l.split()[2]) for l in handle}
                out_dists = {( ".".join(k[0].split(".")[:-1]) if any([k[0].endswith(ext) for ext in FASTA_EXTS]) else k[0],
                               ".".join(k[1].split(".")[:-1]) if any([k[1].endswith(ext) for ext in FASTA_EXTS]) else k[1] ): v
                               for k,v in out_dists.items() }
#                tfile = lambda k : ".".join(k.split(".")[:-1]) if (k.endswith(".fna") or k.endswith(".fa") or k.endswith(".fasta") or k.endswith(".fna") or k.endswith(".ffn")) else k
#                out_dists = {(tfile(k[0]),tfile(k[1])) : v for k,v in out_dists.items()}
            if outfile is None:
                os.remove(fastani_file)
        else :
            print("No other method for ani computation implemented yet")
            sys.exit()

        return out_dists



import setuptools
from mOTUlizer import __version__

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mOTUlizer", # Replace with your own username
    version=__version__,
    author="Moritz Buck",
    author_email="moritz.buck@slu.se",
    description="making OTUs from genomes, and stats on them. and even core-genomes",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/moritzbuck/mOTUlizer/",
    packages= setuptools.find_packages(),
    install_requires = [
    "python-igraph", "biopython"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Bio-Informatics"
    ],
    keywords='bioinformatics clustering metagenomics microbial-genomics genomics',
    python_requires='>=3.6',
    scripts = ['mOTUlizer/bin/mOTUlize.py','mOTUlizer/bin/mOTUpan.py','mOTUlizer/bin/mOTUconvert.py']

)

# pushing stuff to pip : python3 setup.py sdist bdist_wheel
# python3 -m twine upload --repository pypi dist/*



import subprocess
from importlib.util import find_spec
import os

cwd = os.getcwd()
module_path = find_spec("mOTUlizer").submodule_search_locations[0]

os.chdir(module_path)

if os.path.exists(".git"):
    label = subprocess.check_output(["git", "describe", "--tags"]).strip().decode()
else:
    label = "0.2.4"

os.chdir(cwd)

__version__ =  label



#!/usr/bin/env python

import os
import shutil
import sys
from os.path import join as pjoin
import argparse
import json
from mOTUlizer import __version__

from mOTUlizer.utils import *
from mOTUlizer.classes.Parser import EmapperParse, PPanGGolinParse, RoaryParse, MmseqsParse

description_text = """
Converts output of diverse software generatig COGs, or genetically encoded traits into a genome2cog-JSON file useable by mOTUpan
"""

list_text = """
emapper :
\tdescribes each genomes as a set of eggNOGs based on the output of eggNOGmapper,
\tonly the deepest (e.g. closest to the root) eggNOG for each hit is taken,
\tinput-file is just the  '.emapper.annotations'-file

ppanggolin :
\textracts the gene families from the hdf5-file (e.g. '.h5' file in the output folder)

roary :
\tjust converts the cluster output file, clustered_proteins by default, the file
\tof the '-o' switch, to a json file readable by mOTUpan

mmseqs2 :
\tjust converts the cluster output file, <o:cluster_Prefix>_cluster.tsv by default, generated
\twhen running mmseq2 easy-cluster, where <o:cluster_Prefix> is the output prefix you give to
\tthe command.


"""

def motuconvert(args):

    method = args.in_type
    if method == "emapper":
        converter = EmapperParse()
    elif method == "ppanggolin":
        converter = PPanGGolinParse()
    elif method == "roary":
        converter = RoaryParse()
    elif method == "mmseqs2":
        converter = MmseqsParse()
    else :
        print("This is not implemented yet run with '--list' to see available options", file = sys.stderr )
        sys.exit(0)

    genome2cogs = converter.convert(infile = args.input, count = args.count)

    if args.output:
        out_handle = open(out_json, "w")
    else :
        out_handle = sys.stdout

    json.dump(genome2cogs, out_handle, indent=4, sort_keys=True)

    if args.output:
        out_handle.close()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(prog = "mOTUconverts.py", description=description_text, epilog = "Let's do this")
    parser.add_argument('--output', '-o', nargs = '?', help = "send output to this file defaults to stdout")
    parser.add_argument('input', nargs = '?', metavar = "INPUT", help = "input file(s), check '--list' for specifics")
    parser.add_argument('--gene2genome', nargs='?', help = "if gene names not '${genome_name}_[0-9]*', a tab separated file with id of gene in the fist column and a semi-column separated second column containing genomes_id of genomes containing it")
    parser.add_argument('--in_type', '-I', nargs = '?', default = "emapper" , help = "software generating the input")
    parser.add_argument('--version','-v', action="store_true", help = "get version and exit")
    parser.add_argument('--list','-l', action="store_true", help = "list tools available and exit")
    parser.add_argument('--count','-c', action="store_true", default = False, help = "count the occurences of COG/trait")

    args = parser.parse_args()

    if len(sys.argv)==1:
        parser.print_help()
        print("{script} Version {version}".format(script = __file__, version = __version__), file = sys.stderr)
        sys.exit(1)

    if args.version:
        print("{script} Version {version}".format(script = __file__, version = __version__), file = sys.stderr)
        sys.exit()

    if args.list:
        print(list_text)
        print("{script} Version {version}".format(script = __file__, version = __version__), file = sys.stderr)
        sys.exit()

    motuconvert(args)
